// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.Azure.Management.ApiManagement;
using Microsoft.Azure.Management.ApiManagement.Models;
using Newtonsoft.Json.Linq;

namespace Microsoft.Azure.Management.ApiManagement
{
    /// <summary>
    /// Operations for managing Api Management service provisioning
    /// (create/remove, backup/restore, scale, etc.).
    /// </summary>
    internal partial class ResourceProviderOperations : IServiceOperations<ApiManagementClient>, IResourceProviderOperations
    {
        /// <summary>
        /// Initializes a new instance of the ResourceProviderOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal ResourceProviderOperations(ApiManagementClient client)
        {
            this._client = client;
        }
        
        private ApiManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.Azure.Management.ApiManagement.ApiManagementClient.
        /// </summary>
        public ApiManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Backs up an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BackupAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BackupAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginBackupAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Begin backup operation of an Api Management service.To determine
        /// whether the operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the BeginBackup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginBackupAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.AccessKey == null)
            {
                throw new ArgumentNullException("parameters.AccessKey");
            }
            if (parameters.BackupName == null)
            {
                throw new ArgumentNullException("parameters.BackupName");
            }
            if (parameters.ContainerName == null)
            {
                throw new ArgumentNullException("parameters.ContainerName");
            }
            if (parameters.StorageAccount == null)
            {
                throw new ArgumentNullException("parameters.StorageAccount");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginBackupAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/backup";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceBackupRestoreParametersValue = new JObject();
                requestDoc = apiServiceBackupRestoreParametersValue;
                
                apiServiceBackupRestoreParametersValue["storageAccount"] = parameters.StorageAccount;
                
                apiServiceBackupRestoreParametersValue["accessKey"] = parameters.AccessKey;
                
                apiServiceBackupRestoreParametersValue["containerName"] = parameters.ContainerName;
                
                apiServiceBackupRestoreParametersValue["backupName"] = parameters.BackupName;
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new ApiServiceLongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Begins creating new or updating existing Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetApiServiceLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate Api Management
        /// service operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginCreatingOrUpdatingAsync(string resourceGroupName, string name, ApiServiceCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Location == null)
            {
                throw new ArgumentNullException("parameters.Location");
            }
            if (parameters.Properties == null)
            {
                throw new ArgumentNullException("parameters.Properties");
            }
            if (parameters.Properties.HostnameConfigurations != null)
            {
                foreach (HostnameConfiguration hostnameConfigurationsParameterItem in parameters.Properties.HostnameConfigurations)
                {
                    if (hostnameConfigurationsParameterItem.Certificate == null)
                    {
                        throw new ArgumentNullException("parameters.Properties.HostnameConfigurations.Certificate");
                    }
                    if (hostnameConfigurationsParameterItem.Certificate.Subject == null)
                    {
                        throw new ArgumentNullException("parameters.Properties.HostnameConfigurations.Certificate.Subject");
                    }
                    if (hostnameConfigurationsParameterItem.Certificate.Thumbprint == null)
                    {
                        throw new ArgumentNullException("parameters.Properties.HostnameConfigurations.Certificate.Thumbprint");
                    }
                    if (hostnameConfigurationsParameterItem.Hostname == null)
                    {
                        throw new ArgumentNullException("parameters.Properties.HostnameConfigurations.Hostname");
                    }
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginCreatingOrUpdatingAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceCreateOrUpdateParametersValue = new JObject();
                requestDoc = apiServiceCreateOrUpdateParametersValue;
                
                apiServiceCreateOrUpdateParametersValue["location"] = parameters.Location;
                
                if (parameters.Tags != null)
                {
                    if (parameters.Tags is ILazyCollection == false || ((ILazyCollection)parameters.Tags).IsInitialized)
                    {
                        JObject tagsDictionary = new JObject();
                        foreach (KeyValuePair<string, string> pair in parameters.Tags)
                        {
                            string tagsKey = pair.Key;
                            string tagsValue = pair.Value;
                            tagsDictionary[tagsKey] = tagsValue;
                        }
                        apiServiceCreateOrUpdateParametersValue["tags"] = tagsDictionary;
                    }
                }
                
                JObject propertiesValue = new JObject();
                apiServiceCreateOrUpdateParametersValue["properties"] = propertiesValue;
                
                if (parameters.Properties.PublisherEmail != null)
                {
                    propertiesValue["publisherEmail"] = parameters.Properties.PublisherEmail;
                }
                
                if (parameters.Properties.PublisherName != null)
                {
                    propertiesValue["publisherName"] = parameters.Properties.PublisherName;
                }
                
                if (parameters.Properties.SkuProperties != null)
                {
                    JObject skuValue = new JObject();
                    propertiesValue["sku"] = skuValue;
                    
                    skuValue["name"] = parameters.Properties.SkuProperties.SkuType.ToString();
                    
                    if (parameters.Properties.SkuProperties.Capacity != null)
                    {
                        skuValue["capacity"] = parameters.Properties.SkuProperties.Capacity.Value;
                    }
                }
                
                if (parameters.Properties.ProvisioningState != null)
                {
                    propertiesValue["provisioningState"] = parameters.Properties.ProvisioningState;
                }
                
                if (parameters.Properties.TargetProvisioningState != null)
                {
                    propertiesValue["targetProvisioningState"] = parameters.Properties.TargetProvisioningState;
                }
                
                propertiesValue["createdAtUtc"] = parameters.Properties.CreatedAtUtc;
                
                if (parameters.Properties.ProxyEndpoint != null)
                {
                    propertiesValue["runtimeUrl"] = parameters.Properties.ProxyEndpoint;
                }
                
                if (parameters.Properties.ManagementPortalEndpoint != null)
                {
                    propertiesValue["portalUrl"] = parameters.Properties.ManagementPortalEndpoint;
                }
                
                if (parameters.Properties.AddresserEmail != null)
                {
                    propertiesValue["addresserEmail"] = parameters.Properties.AddresserEmail;
                }
                
                if (parameters.Properties.HostnameConfigurations != null)
                {
                    if (parameters.Properties.HostnameConfigurations is ILazyCollection == false || ((ILazyCollection)parameters.Properties.HostnameConfigurations).IsInitialized)
                    {
                        JArray hostnameConfigurationsArray = new JArray();
                        foreach (HostnameConfiguration hostnameConfigurationsItem in parameters.Properties.HostnameConfigurations)
                        {
                            JObject hostnameConfigurationValue = new JObject();
                            hostnameConfigurationsArray.Add(hostnameConfigurationValue);
                            
                            hostnameConfigurationValue["type"] = ((int)hostnameConfigurationsItem.Type);
                            
                            hostnameConfigurationValue["hostname"] = hostnameConfigurationsItem.Hostname;
                            
                            JObject certificateValue = new JObject();
                            hostnameConfigurationValue["certificate"] = certificateValue;
                            
                            certificateValue["expiry"] = hostnameConfigurationsItem.Certificate.Expiry;
                            
                            certificateValue["thumbprint"] = hostnameConfigurationsItem.Certificate.Thumbprint;
                            
                            certificateValue["subject"] = hostnameConfigurationsItem.Certificate.Subject;
                        }
                        propertiesValue["hostnameConfigurations"] = hostnameConfigurationsArray;
                    }
                }
                
                if (parameters.Properties.StaticIPs != null)
                {
                    if (parameters.Properties.StaticIPs is ILazyCollection == false || ((ILazyCollection)parameters.Properties.StaticIPs).IsInitialized)
                    {
                        JArray staticIPsArray = new JArray();
                        foreach (string staticIPsItem in parameters.Properties.StaticIPs)
                        {
                            staticIPsArray.Add(staticIPsItem);
                        }
                        propertiesValue["staticIPs"] = staticIPsArray;
                    }
                }
                
                if (parameters.Properties.VirtualNetworkConfiguration != null)
                {
                    JObject vpnconfigurationValue = new JObject();
                    propertiesValue["vpnconfiguration"] = vpnconfigurationValue;
                    
                    vpnconfigurationValue["vnetid"] = parameters.Properties.VirtualNetworkConfiguration.VnetId.ToString();
                    
                    if (parameters.Properties.VirtualNetworkConfiguration.SubnetName != null)
                    {
                        vpnconfigurationValue["subnetname"] = parameters.Properties.VirtualNetworkConfiguration.SubnetName;
                    }
                    
                    if (parameters.Properties.VirtualNetworkConfiguration.Location != null)
                    {
                        vpnconfigurationValue["location"] = parameters.Properties.VirtualNetworkConfiguration.Location;
                    }
                }
                
                if (parameters.Properties.AdditionalRegions != null)
                {
                    if (parameters.Properties.AdditionalRegions is ILazyCollection == false || ((ILazyCollection)parameters.Properties.AdditionalRegions).IsInitialized)
                    {
                        JArray additionalLocationsArray = new JArray();
                        foreach (AdditionalRegion additionalLocationsItem in parameters.Properties.AdditionalRegions)
                        {
                            JObject additionalRegionValue = new JObject();
                            additionalLocationsArray.Add(additionalRegionValue);
                            
                            if (additionalLocationsItem.Location != null)
                            {
                                additionalRegionValue["location"] = additionalLocationsItem.Location;
                            }
                            
                            additionalRegionValue["skuType"] = additionalLocationsItem.SkuType.ToString();
                            
                            if (additionalLocationsItem.SkuUnitCount != null)
                            {
                                additionalRegionValue["skuUnitCount"] = additionalLocationsItem.SkuUnitCount.Value;
                            }
                            
                            if (additionalLocationsItem.StaticIPs != null)
                            {
                                if (additionalLocationsItem.StaticIPs is ILazyCollection == false || ((ILazyCollection)additionalLocationsItem.StaticIPs).IsInitialized)
                                {
                                    JArray staticIPsArray2 = new JArray();
                                    foreach (string staticIPsItem2 in additionalLocationsItem.StaticIPs)
                                    {
                                        staticIPsArray2.Add(staticIPsItem2);
                                    }
                                    additionalRegionValue["staticIPs"] = staticIPsArray2;
                                }
                            }
                            
                            if (additionalLocationsItem.VirtualNetworkConfiguration != null)
                            {
                                JObject vpnconfigurationValue2 = new JObject();
                                additionalRegionValue["vpnconfiguration"] = vpnconfigurationValue2;
                                
                                vpnconfigurationValue2["vnetid"] = additionalLocationsItem.VirtualNetworkConfiguration.VnetId.ToString();
                                
                                if (additionalLocationsItem.VirtualNetworkConfiguration.SubnetName != null)
                                {
                                    vpnconfigurationValue2["subnetname"] = additionalLocationsItem.VirtualNetworkConfiguration.SubnetName;
                                }
                                
                                if (additionalLocationsItem.VirtualNetworkConfiguration.Location != null)
                                {
                                    vpnconfigurationValue2["location"] = additionalLocationsItem.VirtualNetworkConfiguration.Location;
                                }
                            }
                        }
                        propertiesValue["additionalLocations"] = additionalLocationsArray;
                    }
                }
                
                if (parameters.Properties.CustomProperties != null)
                {
                    if (parameters.Properties.CustomProperties is ILazyCollection == false || ((ILazyCollection)parameters.Properties.CustomProperties).IsInitialized)
                    {
                        JObject customPropertiesDictionary = new JObject();
                        foreach (KeyValuePair<string, string> pair2 in parameters.Properties.CustomProperties)
                        {
                            string customPropertiesKey = pair2.Key;
                            string customPropertiesValue = pair2.Value;
                            customPropertiesDictionary[customPropertiesKey] = customPropertiesValue;
                        }
                        propertiesValue["customProperties"] = customPropertiesDictionary;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Created && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Created || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceLongRunningOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ApiServiceResource valueInstance = new ApiServiceResource();
                            result.Value = valueInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                valueInstance.Id = idInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                valueInstance.Location = locationInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                valueInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                valueInstance.Type = typeInstance;
                            }
                            
                            JToken etagValue = responseDoc["etag"];
                            if (etagValue != null && etagValue.Type != JTokenType.Null)
                            {
                                string etagInstance = ((string)etagValue);
                                valueInstance.ETag = etagInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey2 = ((string)property.Name);
                                    string tagsValue2 = ((string)property.Value);
                                    valueInstance.Tags.Add(tagsKey2, tagsValue2);
                                }
                            }
                            
                            JToken propertiesValue2 = responseDoc["properties"];
                            if (propertiesValue2 != null && propertiesValue2.Type != JTokenType.Null)
                            {
                                ApiServiceProperties propertiesInstance = new ApiServiceProperties();
                                valueInstance.Properties = propertiesInstance;
                                
                                JToken publisherEmailValue = propertiesValue2["publisherEmail"];
                                if (publisherEmailValue != null && publisherEmailValue.Type != JTokenType.Null)
                                {
                                    string publisherEmailInstance = ((string)publisherEmailValue);
                                    propertiesInstance.PublisherEmail = publisherEmailInstance;
                                }
                                
                                JToken publisherNameValue = propertiesValue2["publisherName"];
                                if (publisherNameValue != null && publisherNameValue.Type != JTokenType.Null)
                                {
                                    string publisherNameInstance = ((string)publisherNameValue);
                                    propertiesInstance.PublisherName = publisherNameInstance;
                                }
                                
                                JToken skuValue2 = propertiesValue2["sku"];
                                if (skuValue2 != null && skuValue2.Type != JTokenType.Null)
                                {
                                    ApiServiceSkuProperties skuInstance = new ApiServiceSkuProperties();
                                    propertiesInstance.SkuProperties = skuInstance;
                                    
                                    JToken nameValue2 = skuValue2["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        SkuType nameInstance2 = ((SkuType)Enum.Parse(typeof(SkuType), ((string)nameValue2), true));
                                        skuInstance.SkuType = nameInstance2;
                                    }
                                    
                                    JToken capacityValue = skuValue2["capacity"];
                                    if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                    {
                                        int capacityInstance = ((int)capacityValue);
                                        skuInstance.Capacity = capacityInstance;
                                    }
                                }
                                
                                JToken provisioningStateValue = propertiesValue2["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken targetProvisioningStateValue = propertiesValue2["targetProvisioningState"];
                                if (targetProvisioningStateValue != null && targetProvisioningStateValue.Type != JTokenType.Null)
                                {
                                    string targetProvisioningStateInstance = ((string)targetProvisioningStateValue);
                                    propertiesInstance.TargetProvisioningState = targetProvisioningStateInstance;
                                }
                                
                                JToken createdAtUtcValue = propertiesValue2["createdAtUtc"];
                                if (createdAtUtcValue != null && createdAtUtcValue.Type != JTokenType.Null)
                                {
                                    DateTime createdAtUtcInstance = ((DateTime)createdAtUtcValue);
                                    propertiesInstance.CreatedAtUtc = createdAtUtcInstance;
                                }
                                
                                JToken runtimeUrlValue = propertiesValue2["runtimeUrl"];
                                if (runtimeUrlValue != null && runtimeUrlValue.Type != JTokenType.Null)
                                {
                                    string runtimeUrlInstance = ((string)runtimeUrlValue);
                                    propertiesInstance.ProxyEndpoint = runtimeUrlInstance;
                                }
                                
                                JToken portalUrlValue = propertiesValue2["portalUrl"];
                                if (portalUrlValue != null && portalUrlValue.Type != JTokenType.Null)
                                {
                                    string portalUrlInstance = ((string)portalUrlValue);
                                    propertiesInstance.ManagementPortalEndpoint = portalUrlInstance;
                                }
                                
                                JToken addresserEmailValue = propertiesValue2["addresserEmail"];
                                if (addresserEmailValue != null && addresserEmailValue.Type != JTokenType.Null)
                                {
                                    string addresserEmailInstance = ((string)addresserEmailValue);
                                    propertiesInstance.AddresserEmail = addresserEmailInstance;
                                }
                                
                                JToken hostnameConfigurationsArray2 = propertiesValue2["hostnameConfigurations"];
                                if (hostnameConfigurationsArray2 != null && hostnameConfigurationsArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken hostnameConfigurationsValue in ((JArray)hostnameConfigurationsArray2))
                                    {
                                        HostnameConfiguration hostnameConfigurationInstance = new HostnameConfiguration();
                                        propertiesInstance.HostnameConfigurations.Add(hostnameConfigurationInstance);
                                        
                                        JToken typeValue2 = hostnameConfigurationsValue["type"];
                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                        {
                                            HostnameType typeInstance2 = ((HostnameType)(((int)typeValue2)));
                                            hostnameConfigurationInstance.Type = typeInstance2;
                                        }
                                        
                                        JToken hostnameValue = hostnameConfigurationsValue["hostname"];
                                        if (hostnameValue != null && hostnameValue.Type != JTokenType.Null)
                                        {
                                            string hostnameInstance = ((string)hostnameValue);
                                            hostnameConfigurationInstance.Hostname = hostnameInstance;
                                        }
                                        
                                        JToken certificateValue2 = hostnameConfigurationsValue["certificate"];
                                        if (certificateValue2 != null && certificateValue2.Type != JTokenType.Null)
                                        {
                                            CertificateInformation certificateInstance = new CertificateInformation();
                                            hostnameConfigurationInstance.Certificate = certificateInstance;
                                            
                                            JToken expiryValue = certificateValue2["expiry"];
                                            if (expiryValue != null && expiryValue.Type != JTokenType.Null)
                                            {
                                                DateTime expiryInstance = ((DateTime)expiryValue);
                                                certificateInstance.Expiry = expiryInstance;
                                            }
                                            
                                            JToken thumbprintValue = certificateValue2["thumbprint"];
                                            if (thumbprintValue != null && thumbprintValue.Type != JTokenType.Null)
                                            {
                                                string thumbprintInstance = ((string)thumbprintValue);
                                                certificateInstance.Thumbprint = thumbprintInstance;
                                            }
                                            
                                            JToken subjectValue = certificateValue2["subject"];
                                            if (subjectValue != null && subjectValue.Type != JTokenType.Null)
                                            {
                                                string subjectInstance = ((string)subjectValue);
                                                certificateInstance.Subject = subjectInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken staticIPsArray3 = propertiesValue2["staticIPs"];
                                if (staticIPsArray3 != null && staticIPsArray3.Type != JTokenType.Null)
                                {
                                    foreach (JToken staticIPsValue in ((JArray)staticIPsArray3))
                                    {
                                        propertiesInstance.StaticIPs.Add(((string)staticIPsValue));
                                    }
                                }
                                
                                JToken vpnconfigurationValue3 = propertiesValue2["vpnconfiguration"];
                                if (vpnconfigurationValue3 != null && vpnconfigurationValue3.Type != JTokenType.Null)
                                {
                                    VirtualNetworkConfiguration vpnconfigurationInstance = new VirtualNetworkConfiguration();
                                    propertiesInstance.VirtualNetworkConfiguration = vpnconfigurationInstance;
                                    
                                    JToken vnetidValue = vpnconfigurationValue3["vnetid"];
                                    if (vnetidValue != null && vnetidValue.Type != JTokenType.Null)
                                    {
                                        Guid vnetidInstance = Guid.Parse(((string)vnetidValue));
                                        vpnconfigurationInstance.VnetId = vnetidInstance;
                                    }
                                    
                                    JToken subnetnameValue = vpnconfigurationValue3["subnetname"];
                                    if (subnetnameValue != null && subnetnameValue.Type != JTokenType.Null)
                                    {
                                        string subnetnameInstance = ((string)subnetnameValue);
                                        vpnconfigurationInstance.SubnetName = subnetnameInstance;
                                    }
                                    
                                    JToken locationValue2 = vpnconfigurationValue3["location"];
                                    if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                                    {
                                        string locationInstance2 = ((string)locationValue2);
                                        vpnconfigurationInstance.Location = locationInstance2;
                                    }
                                }
                                
                                JToken additionalLocationsArray2 = propertiesValue2["additionalLocations"];
                                if (additionalLocationsArray2 != null && additionalLocationsArray2.Type != JTokenType.Null)
                                {
                                    foreach (JToken additionalLocationsValue in ((JArray)additionalLocationsArray2))
                                    {
                                        AdditionalRegion additionalRegionInstance = new AdditionalRegion();
                                        propertiesInstance.AdditionalRegions.Add(additionalRegionInstance);
                                        
                                        JToken locationValue3 = additionalLocationsValue["location"];
                                        if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                                        {
                                            string locationInstance3 = ((string)locationValue3);
                                            additionalRegionInstance.Location = locationInstance3;
                                        }
                                        
                                        JToken skuTypeValue = additionalLocationsValue["skuType"];
                                        if (skuTypeValue != null && skuTypeValue.Type != JTokenType.Null)
                                        {
                                            SkuType skuTypeInstance = ((SkuType)Enum.Parse(typeof(SkuType), ((string)skuTypeValue), true));
                                            additionalRegionInstance.SkuType = skuTypeInstance;
                                        }
                                        
                                        JToken skuUnitCountValue = additionalLocationsValue["skuUnitCount"];
                                        if (skuUnitCountValue != null && skuUnitCountValue.Type != JTokenType.Null)
                                        {
                                            int skuUnitCountInstance = ((int)skuUnitCountValue);
                                            additionalRegionInstance.SkuUnitCount = skuUnitCountInstance;
                                        }
                                        
                                        JToken staticIPsArray4 = additionalLocationsValue["staticIPs"];
                                        if (staticIPsArray4 != null && staticIPsArray4.Type != JTokenType.Null)
                                        {
                                            foreach (JToken staticIPsValue2 in ((JArray)staticIPsArray4))
                                            {
                                                additionalRegionInstance.StaticIPs.Add(((string)staticIPsValue2));
                                            }
                                        }
                                        
                                        JToken vpnconfigurationValue4 = additionalLocationsValue["vpnconfiguration"];
                                        if (vpnconfigurationValue4 != null && vpnconfigurationValue4.Type != JTokenType.Null)
                                        {
                                            VirtualNetworkConfiguration vpnconfigurationInstance2 = new VirtualNetworkConfiguration();
                                            additionalRegionInstance.VirtualNetworkConfiguration = vpnconfigurationInstance2;
                                            
                                            JToken vnetidValue2 = vpnconfigurationValue4["vnetid"];
                                            if (vnetidValue2 != null && vnetidValue2.Type != JTokenType.Null)
                                            {
                                                Guid vnetidInstance2 = Guid.Parse(((string)vnetidValue2));
                                                vpnconfigurationInstance2.VnetId = vnetidInstance2;
                                            }
                                            
                                            JToken subnetnameValue2 = vpnconfigurationValue4["subnetname"];
                                            if (subnetnameValue2 != null && subnetnameValue2.Type != JTokenType.Null)
                                            {
                                                string subnetnameInstance2 = ((string)subnetnameValue2);
                                                vpnconfigurationInstance2.SubnetName = subnetnameInstance2;
                                            }
                                            
                                            JToken locationValue4 = vpnconfigurationValue4["location"];
                                            if (locationValue4 != null && locationValue4.Type != JTokenType.Null)
                                            {
                                                string locationInstance4 = ((string)locationValue4);
                                                vpnconfigurationInstance2.Location = locationInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken customPropertiesSequenceElement = ((JToken)propertiesValue2["customProperties"]);
                                if (customPropertiesSequenceElement != null && customPropertiesSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property2 in customPropertiesSequenceElement)
                                    {
                                        string customPropertiesKey2 = ((string)property2.Name);
                                        string customPropertiesValue2 = ((string)property2.Value);
                                        propertiesInstance.CustomProperties.Add(customPropertiesKey2, customPropertiesValue2);
                                    }
                                }
                            }
                            
                            ApiManagementError errorInstance = new ApiManagementError();
                            result.Error = errorInstance;
                            
                            JToken codeValue = responseDoc["code"];
                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                            {
                                string codeInstance = ((string)codeValue);
                                errorInstance.Code = codeInstance;
                            }
                            
                            JToken messageValue = responseDoc["message"];
                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                            {
                                string messageInstance = ((string)messageValue);
                                errorInstance.Message = messageInstance;
                            }
                            
                            JToken detailsSequenceElement = ((JToken)responseDoc["details"]);
                            if (detailsSequenceElement != null && detailsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property3 in detailsSequenceElement)
                                {
                                    string detailsKey = ((string)property3.Name);
                                    string detailsValue = ((string)property3.Value);
                                    errorInstance.Details.Add(detailsKey, detailsValue);
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Begin to manage (CUD) deployments of an Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the ManageDeployments operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginManagingDeploymentsAsync(string resourceGroupName, string name, ApiServiceManageDeploymentsParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Location == null)
            {
                throw new ArgumentNullException("parameters.Location");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginManagingDeploymentsAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/managedeployments";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceManageDeploymentsParametersValue = new JObject();
                requestDoc = apiServiceManageDeploymentsParametersValue;
                
                apiServiceManageDeploymentsParametersValue["location"] = parameters.Location;
                
                apiServiceManageDeploymentsParametersValue["skuType"] = parameters.SkuType.ToString();
                
                if (parameters.SkuUnitCount != null)
                {
                    apiServiceManageDeploymentsParametersValue["skuUnitCount"] = parameters.SkuUnitCount.Value;
                }
                
                if (parameters.AdditionalRegions != null)
                {
                    if (parameters.AdditionalRegions is ILazyCollection == false || ((ILazyCollection)parameters.AdditionalRegions).IsInitialized)
                    {
                        JArray additionalLocationsArray = new JArray();
                        foreach (AdditionalRegion additionalLocationsItem in parameters.AdditionalRegions)
                        {
                            JObject additionalRegionValue = new JObject();
                            additionalLocationsArray.Add(additionalRegionValue);
                            
                            if (additionalLocationsItem.Location != null)
                            {
                                additionalRegionValue["location"] = additionalLocationsItem.Location;
                            }
                            
                            additionalRegionValue["skuType"] = additionalLocationsItem.SkuType.ToString();
                            
                            if (additionalLocationsItem.SkuUnitCount != null)
                            {
                                additionalRegionValue["skuUnitCount"] = additionalLocationsItem.SkuUnitCount.Value;
                            }
                            
                            if (additionalLocationsItem.StaticIPs != null)
                            {
                                if (additionalLocationsItem.StaticIPs is ILazyCollection == false || ((ILazyCollection)additionalLocationsItem.StaticIPs).IsInitialized)
                                {
                                    JArray staticIPsArray = new JArray();
                                    foreach (string staticIPsItem in additionalLocationsItem.StaticIPs)
                                    {
                                        staticIPsArray.Add(staticIPsItem);
                                    }
                                    additionalRegionValue["staticIPs"] = staticIPsArray;
                                }
                            }
                            
                            if (additionalLocationsItem.VirtualNetworkConfiguration != null)
                            {
                                JObject vpnconfigurationValue = new JObject();
                                additionalRegionValue["vpnconfiguration"] = vpnconfigurationValue;
                                
                                vpnconfigurationValue["vnetid"] = additionalLocationsItem.VirtualNetworkConfiguration.VnetId.ToString();
                                
                                if (additionalLocationsItem.VirtualNetworkConfiguration.SubnetName != null)
                                {
                                    vpnconfigurationValue["subnetname"] = additionalLocationsItem.VirtualNetworkConfiguration.SubnetName;
                                }
                                
                                if (additionalLocationsItem.VirtualNetworkConfiguration.Location != null)
                                {
                                    vpnconfigurationValue["location"] = additionalLocationsItem.VirtualNetworkConfiguration.Location;
                                }
                            }
                        }
                        apiServiceManageDeploymentsParametersValue["additionalLocations"] = additionalLocationsArray;
                    }
                }
                
                if (parameters.VirtualNetworkConfiguration != null)
                {
                    JObject vpnConfigurationValue = new JObject();
                    apiServiceManageDeploymentsParametersValue["vpnConfiguration"] = vpnConfigurationValue;
                    
                    vpnConfigurationValue["vnetid"] = parameters.VirtualNetworkConfiguration.VnetId.ToString();
                    
                    if (parameters.VirtualNetworkConfiguration.SubnetName != null)
                    {
                        vpnConfigurationValue["subnetname"] = parameters.VirtualNetworkConfiguration.SubnetName;
                    }
                    
                    if (parameters.VirtualNetworkConfiguration.Location != null)
                    {
                        vpnConfigurationValue["location"] = parameters.VirtualNetworkConfiguration.Location;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new ApiServiceLongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Begin to manage (CUD) VPN configuration of an Api Management
        /// service.To determine whether the operation has finished processing
        /// the request, call GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the ManageVirtualNetworks
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginManagingVirtualNetworksAsync(string resourceGroupName, string name, ApiServiceManageVirtualNetworksParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginManagingVirtualNetworksAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/managevpn";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceManageVirtualNetworksParametersValue = new JObject();
                requestDoc = apiServiceManageVirtualNetworksParametersValue;
                
                if (parameters.VirtualNetworkConfigurations != null)
                {
                    if (parameters.VirtualNetworkConfigurations is ILazyCollection == false || ((ILazyCollection)parameters.VirtualNetworkConfigurations).IsInitialized)
                    {
                        JArray vpnconfigurationsArray = new JArray();
                        foreach (VirtualNetworkConfiguration vpnconfigurationsItem in parameters.VirtualNetworkConfigurations)
                        {
                            JObject virtualNetworkConfigurationValue = new JObject();
                            vpnconfigurationsArray.Add(virtualNetworkConfigurationValue);
                            
                            virtualNetworkConfigurationValue["vnetid"] = vpnconfigurationsItem.VnetId.ToString();
                            
                            if (vpnconfigurationsItem.SubnetName != null)
                            {
                                virtualNetworkConfigurationValue["subnetname"] = vpnconfigurationsItem.SubnetName;
                            }
                            
                            if (vpnconfigurationsItem.Location != null)
                            {
                                virtualNetworkConfigurationValue["location"] = vpnconfigurationsItem.Location;
                            }
                        }
                        apiServiceManageVirtualNetworksParametersValue["vpnconfigurations"] = vpnconfigurationsArray;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new ApiServiceLongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Begin restore from backup operation of an Api Management service.To
        /// determine whether the operation has finished processing the
        /// request, call GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Restore Api Management service
        /// from backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginRestoringAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.AccessKey == null)
            {
                throw new ArgumentNullException("parameters.AccessKey");
            }
            if (parameters.BackupName == null)
            {
                throw new ArgumentNullException("parameters.BackupName");
            }
            if (parameters.ContainerName == null)
            {
                throw new ArgumentNullException("parameters.ContainerName");
            }
            if (parameters.StorageAccount == null)
            {
                throw new ArgumentNullException("parameters.StorageAccount");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginRestoringAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/restore";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceBackupRestoreParametersValue = new JObject();
                requestDoc = apiServiceBackupRestoreParametersValue;
                
                apiServiceBackupRestoreParametersValue["storageAccount"] = parameters.StorageAccount;
                
                apiServiceBackupRestoreParametersValue["accessKey"] = parameters.AccessKey;
                
                apiServiceBackupRestoreParametersValue["containerName"] = parameters.ContainerName;
                
                apiServiceBackupRestoreParametersValue["backupName"] = parameters.BackupName;
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new ApiServiceLongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Begin updating hostname of an Api Management service.To determine
        /// whether the operation has finished processing the request, call
        /// GetLongRunningOperationStatus.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the UpdateHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> BeginUpdatingHostnameAsync(string resourceGroupName, string name, ApiServiceUpdateHostnameParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.HostnamesToCreateOrUpdate != null)
            {
                foreach (HostnameConfiguration hostnamesToCreateOrUpdateParameterItem in parameters.HostnamesToCreateOrUpdate)
                {
                    if (hostnamesToCreateOrUpdateParameterItem.Certificate == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCreateOrUpdate.Certificate");
                    }
                    if (hostnamesToCreateOrUpdateParameterItem.Certificate.Subject == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCreateOrUpdate.Certificate.Subject");
                    }
                    if (hostnamesToCreateOrUpdateParameterItem.Certificate.Thumbprint == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCreateOrUpdate.Certificate.Thumbprint");
                    }
                    if (hostnamesToCreateOrUpdateParameterItem.Hostname == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCreateOrUpdate.Hostname");
                    }
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "BeginUpdatingHostnameAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/updatehostname";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceUpdateHostnameParametersValue = new JObject();
                requestDoc = apiServiceUpdateHostnameParametersValue;
                
                if (parameters.HostnamesToCreateOrUpdate != null)
                {
                    if (parameters.HostnamesToCreateOrUpdate is ILazyCollection == false || ((ILazyCollection)parameters.HostnamesToCreateOrUpdate).IsInitialized)
                    {
                        JArray updateArray = new JArray();
                        foreach (HostnameConfiguration updateItem in parameters.HostnamesToCreateOrUpdate)
                        {
                            JObject hostnameConfigurationValue = new JObject();
                            updateArray.Add(hostnameConfigurationValue);
                            
                            hostnameConfigurationValue["type"] = ((int)updateItem.Type);
                            
                            hostnameConfigurationValue["hostname"] = updateItem.Hostname;
                            
                            JObject certificateValue = new JObject();
                            hostnameConfigurationValue["certificate"] = certificateValue;
                            
                            certificateValue["expiry"] = updateItem.Certificate.Expiry;
                            
                            certificateValue["thumbprint"] = updateItem.Certificate.Thumbprint;
                            
                            certificateValue["subject"] = updateItem.Certificate.Subject;
                        }
                        apiServiceUpdateHostnameParametersValue["update"] = updateArray;
                    }
                }
                
                if (parameters.HostnamesToDelete != null)
                {
                    if (parameters.HostnamesToDelete is ILazyCollection == false || ((ILazyCollection)parameters.HostnamesToDelete).IsInitialized)
                    {
                        JArray deleteArray = new JArray();
                        foreach (HostnameType deleteItem in parameters.HostnamesToDelete)
                        {
                            deleteArray.Add(((int)deleteItem));
                        }
                        apiServiceUpdateHostnameParametersValue["delete"] = deleteArray;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new ApiServiceLongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Checks whether the custom host name maps to an Api Management
        /// service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CheckCustomHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CheckCustomHostname operation.
        /// </returns>
        public async Task<ApiServiceCheckCustomHostnameResponse> CheckCustomHostnameAsync(string resourceGroupName, string name, ApiServiceCheckCustomHostnameParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.HostnamesToCheck != null)
            {
                foreach (HostnameConfiguration hostnamesToCheckParameterItem in parameters.HostnamesToCheck)
                {
                    if (hostnamesToCheckParameterItem.Certificate == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCheck.Certificate");
                    }
                    if (hostnamesToCheckParameterItem.Certificate.Subject == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCheck.Certificate.Subject");
                    }
                    if (hostnamesToCheckParameterItem.Certificate.Thumbprint == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCheck.Certificate.Thumbprint");
                    }
                    if (hostnamesToCheckParameterItem.Hostname == null)
                    {
                        throw new ArgumentNullException("parameters.HostnamesToCheck.Hostname");
                    }
                }
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CheckCustomHostnameAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/checkcustomhostname";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceCheckCustomHostnameParametersValue = new JObject();
                requestDoc = apiServiceCheckCustomHostnameParametersValue;
                
                if (parameters.HostnamesToCheck != null)
                {
                    if (parameters.HostnamesToCheck is ILazyCollection == false || ((ILazyCollection)parameters.HostnamesToCheck).IsInitialized)
                    {
                        JArray updateArray = new JArray();
                        foreach (HostnameConfiguration updateItem in parameters.HostnamesToCheck)
                        {
                            JObject hostnameConfigurationValue = new JObject();
                            updateArray.Add(hostnameConfigurationValue);
                            
                            hostnameConfigurationValue["type"] = ((int)updateItem.Type);
                            
                            hostnameConfigurationValue["hostname"] = updateItem.Hostname;
                            
                            JObject certificateValue = new JObject();
                            hostnameConfigurationValue["certificate"] = certificateValue;
                            
                            certificateValue["expiry"] = updateItem.Certificate.Expiry;
                            
                            certificateValue["thumbprint"] = updateItem.Certificate.Thumbprint;
                            
                            certificateValue["subject"] = updateItem.Certificate.Subject;
                        }
                        apiServiceCheckCustomHostnameParametersValue["update"] = updateArray;
                    }
                }
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceCheckCustomHostnameResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceCheckCustomHostnameResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["Value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    CheckCustomHostnameResult checkCustomHostnameResultInstance = new CheckCustomHostnameResult();
                                    result.Value.Add(checkCustomHostnameResultInstance);
                                    
                                    JToken hostnameValue = valueValue["hostname"];
                                    if (hostnameValue != null && hostnameValue.Type != JTokenType.Null)
                                    {
                                        string hostnameInstance = ((string)hostnameValue);
                                        checkCustomHostnameResultInstance.Hostname = hostnameInstance;
                                    }
                                    
                                    JToken isMappedValue = valueValue["isMapped"];
                                    if (isMappedValue != null && isMappedValue.Type != JTokenType.Null)
                                    {
                                        bool isMappedInstance = ((bool)isMappedValue);
                                        checkCustomHostnameResultInstance.IsMapped = isMappedInstance;
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Checks availability and correctness of a name for an Api Management
        /// service.
        /// </summary>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CheckServiceNameAvailability
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Response of the CheckServiceNameAvailability operation.
        /// </returns>
        public async Task<ApiServiceCheckNameAvailabilityResponse> CheckServiceNameAvailabilityAsync(ApiServiceCheckNameAvailabilityParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.Name == null)
            {
                throw new ArgumentNullException("parameters.Name");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CheckServiceNameAvailabilityAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/providers/Microsoft.ApiManagement/checkServiceNameAvailability";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceCheckNameAvailabilityParametersValue = new JObject();
                requestDoc = apiServiceCheckNameAvailabilityParametersValue;
                
                apiServiceCheckNameAvailabilityParametersValue["name"] = parameters.Name;
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceCheckNameAvailabilityResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceCheckNameAvailabilityResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken isAvailableValue = responseDoc["isAvailable"];
                            if (isAvailableValue != null && isAvailableValue.Type != JTokenType.Null)
                            {
                                bool isAvailableInstance = ((bool)isAvailableValue);
                                result.IsAvailable = isAvailableInstance;
                            }
                            
                            JToken reasonValue = responseDoc["reason"];
                            if (reasonValue != null && reasonValue.Type != JTokenType.Null)
                            {
                                string reasonInstance = ((string)reasonValue);
                                result.Reason = reasonInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Creates new or updates existing Api Management service
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the CreateOrUpdate Api Management
        /// service operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> CreateOrUpdateAsync(string resourceGroupName, string name, ApiServiceCreateOrUpdateParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "CreateOrUpdateAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginCreatingOrUpdatingAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Deletes existing Api Management service
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> DeleteAsync(string resourceGroupName, string name, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.NoContent)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureOperationResponse result = null;
                    // Deserialize Response
                    result = new AzureOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Get an Api Management service resource description.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the Get Api Management service operation.
        /// </returns>
        public async Task<ApiServiceGetResponse> GetAsync(string resourceGroupName, string name, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceGetResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceGetResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ApiServiceResource valueInstance = new ApiServiceResource();
                            result.Value = valueInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                valueInstance.Id = idInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                valueInstance.Location = locationInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                valueInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                valueInstance.Type = typeInstance;
                            }
                            
                            JToken etagValue = responseDoc["etag"];
                            if (etagValue != null && etagValue.Type != JTokenType.Null)
                            {
                                string etagInstance = ((string)etagValue);
                                valueInstance.ETag = etagInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    valueInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ApiServiceProperties propertiesInstance = new ApiServiceProperties();
                                valueInstance.Properties = propertiesInstance;
                                
                                JToken publisherEmailValue = propertiesValue["publisherEmail"];
                                if (publisherEmailValue != null && publisherEmailValue.Type != JTokenType.Null)
                                {
                                    string publisherEmailInstance = ((string)publisherEmailValue);
                                    propertiesInstance.PublisherEmail = publisherEmailInstance;
                                }
                                
                                JToken publisherNameValue = propertiesValue["publisherName"];
                                if (publisherNameValue != null && publisherNameValue.Type != JTokenType.Null)
                                {
                                    string publisherNameInstance = ((string)publisherNameValue);
                                    propertiesInstance.PublisherName = publisherNameInstance;
                                }
                                
                                JToken skuValue = propertiesValue["sku"];
                                if (skuValue != null && skuValue.Type != JTokenType.Null)
                                {
                                    ApiServiceSkuProperties skuInstance = new ApiServiceSkuProperties();
                                    propertiesInstance.SkuProperties = skuInstance;
                                    
                                    JToken nameValue2 = skuValue["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        SkuType nameInstance2 = ((SkuType)Enum.Parse(typeof(SkuType), ((string)nameValue2), true));
                                        skuInstance.SkuType = nameInstance2;
                                    }
                                    
                                    JToken capacityValue = skuValue["capacity"];
                                    if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                    {
                                        int capacityInstance = ((int)capacityValue);
                                        skuInstance.Capacity = capacityInstance;
                                    }
                                }
                                
                                JToken provisioningStateValue = propertiesValue["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken targetProvisioningStateValue = propertiesValue["targetProvisioningState"];
                                if (targetProvisioningStateValue != null && targetProvisioningStateValue.Type != JTokenType.Null)
                                {
                                    string targetProvisioningStateInstance = ((string)targetProvisioningStateValue);
                                    propertiesInstance.TargetProvisioningState = targetProvisioningStateInstance;
                                }
                                
                                JToken createdAtUtcValue = propertiesValue["createdAtUtc"];
                                if (createdAtUtcValue != null && createdAtUtcValue.Type != JTokenType.Null)
                                {
                                    DateTime createdAtUtcInstance = ((DateTime)createdAtUtcValue);
                                    propertiesInstance.CreatedAtUtc = createdAtUtcInstance;
                                }
                                
                                JToken runtimeUrlValue = propertiesValue["runtimeUrl"];
                                if (runtimeUrlValue != null && runtimeUrlValue.Type != JTokenType.Null)
                                {
                                    string runtimeUrlInstance = ((string)runtimeUrlValue);
                                    propertiesInstance.ProxyEndpoint = runtimeUrlInstance;
                                }
                                
                                JToken portalUrlValue = propertiesValue["portalUrl"];
                                if (portalUrlValue != null && portalUrlValue.Type != JTokenType.Null)
                                {
                                    string portalUrlInstance = ((string)portalUrlValue);
                                    propertiesInstance.ManagementPortalEndpoint = portalUrlInstance;
                                }
                                
                                JToken addresserEmailValue = propertiesValue["addresserEmail"];
                                if (addresserEmailValue != null && addresserEmailValue.Type != JTokenType.Null)
                                {
                                    string addresserEmailInstance = ((string)addresserEmailValue);
                                    propertiesInstance.AddresserEmail = addresserEmailInstance;
                                }
                                
                                JToken hostnameConfigurationsArray = propertiesValue["hostnameConfigurations"];
                                if (hostnameConfigurationsArray != null && hostnameConfigurationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken hostnameConfigurationsValue in ((JArray)hostnameConfigurationsArray))
                                    {
                                        HostnameConfiguration hostnameConfigurationInstance = new HostnameConfiguration();
                                        propertiesInstance.HostnameConfigurations.Add(hostnameConfigurationInstance);
                                        
                                        JToken typeValue2 = hostnameConfigurationsValue["type"];
                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                        {
                                            HostnameType typeInstance2 = ((HostnameType)(((int)typeValue2)));
                                            hostnameConfigurationInstance.Type = typeInstance2;
                                        }
                                        
                                        JToken hostnameValue = hostnameConfigurationsValue["hostname"];
                                        if (hostnameValue != null && hostnameValue.Type != JTokenType.Null)
                                        {
                                            string hostnameInstance = ((string)hostnameValue);
                                            hostnameConfigurationInstance.Hostname = hostnameInstance;
                                        }
                                        
                                        JToken certificateValue = hostnameConfigurationsValue["certificate"];
                                        if (certificateValue != null && certificateValue.Type != JTokenType.Null)
                                        {
                                            CertificateInformation certificateInstance = new CertificateInformation();
                                            hostnameConfigurationInstance.Certificate = certificateInstance;
                                            
                                            JToken expiryValue = certificateValue["expiry"];
                                            if (expiryValue != null && expiryValue.Type != JTokenType.Null)
                                            {
                                                DateTime expiryInstance = ((DateTime)expiryValue);
                                                certificateInstance.Expiry = expiryInstance;
                                            }
                                            
                                            JToken thumbprintValue = certificateValue["thumbprint"];
                                            if (thumbprintValue != null && thumbprintValue.Type != JTokenType.Null)
                                            {
                                                string thumbprintInstance = ((string)thumbprintValue);
                                                certificateInstance.Thumbprint = thumbprintInstance;
                                            }
                                            
                                            JToken subjectValue = certificateValue["subject"];
                                            if (subjectValue != null && subjectValue.Type != JTokenType.Null)
                                            {
                                                string subjectInstance = ((string)subjectValue);
                                                certificateInstance.Subject = subjectInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken staticIPsArray = propertiesValue["staticIPs"];
                                if (staticIPsArray != null && staticIPsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken staticIPsValue in ((JArray)staticIPsArray))
                                    {
                                        propertiesInstance.StaticIPs.Add(((string)staticIPsValue));
                                    }
                                }
                                
                                JToken vpnconfigurationValue = propertiesValue["vpnconfiguration"];
                                if (vpnconfigurationValue != null && vpnconfigurationValue.Type != JTokenType.Null)
                                {
                                    VirtualNetworkConfiguration vpnconfigurationInstance = new VirtualNetworkConfiguration();
                                    propertiesInstance.VirtualNetworkConfiguration = vpnconfigurationInstance;
                                    
                                    JToken vnetidValue = vpnconfigurationValue["vnetid"];
                                    if (vnetidValue != null && vnetidValue.Type != JTokenType.Null)
                                    {
                                        Guid vnetidInstance = Guid.Parse(((string)vnetidValue));
                                        vpnconfigurationInstance.VnetId = vnetidInstance;
                                    }
                                    
                                    JToken subnetnameValue = vpnconfigurationValue["subnetname"];
                                    if (subnetnameValue != null && subnetnameValue.Type != JTokenType.Null)
                                    {
                                        string subnetnameInstance = ((string)subnetnameValue);
                                        vpnconfigurationInstance.SubnetName = subnetnameInstance;
                                    }
                                    
                                    JToken locationValue2 = vpnconfigurationValue["location"];
                                    if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                                    {
                                        string locationInstance2 = ((string)locationValue2);
                                        vpnconfigurationInstance.Location = locationInstance2;
                                    }
                                }
                                
                                JToken additionalLocationsArray = propertiesValue["additionalLocations"];
                                if (additionalLocationsArray != null && additionalLocationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken additionalLocationsValue in ((JArray)additionalLocationsArray))
                                    {
                                        AdditionalRegion additionalRegionInstance = new AdditionalRegion();
                                        propertiesInstance.AdditionalRegions.Add(additionalRegionInstance);
                                        
                                        JToken locationValue3 = additionalLocationsValue["location"];
                                        if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                                        {
                                            string locationInstance3 = ((string)locationValue3);
                                            additionalRegionInstance.Location = locationInstance3;
                                        }
                                        
                                        JToken skuTypeValue = additionalLocationsValue["skuType"];
                                        if (skuTypeValue != null && skuTypeValue.Type != JTokenType.Null)
                                        {
                                            SkuType skuTypeInstance = ((SkuType)Enum.Parse(typeof(SkuType), ((string)skuTypeValue), true));
                                            additionalRegionInstance.SkuType = skuTypeInstance;
                                        }
                                        
                                        JToken skuUnitCountValue = additionalLocationsValue["skuUnitCount"];
                                        if (skuUnitCountValue != null && skuUnitCountValue.Type != JTokenType.Null)
                                        {
                                            int skuUnitCountInstance = ((int)skuUnitCountValue);
                                            additionalRegionInstance.SkuUnitCount = skuUnitCountInstance;
                                        }
                                        
                                        JToken staticIPsArray2 = additionalLocationsValue["staticIPs"];
                                        if (staticIPsArray2 != null && staticIPsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken staticIPsValue2 in ((JArray)staticIPsArray2))
                                            {
                                                additionalRegionInstance.StaticIPs.Add(((string)staticIPsValue2));
                                            }
                                        }
                                        
                                        JToken vpnconfigurationValue2 = additionalLocationsValue["vpnconfiguration"];
                                        if (vpnconfigurationValue2 != null && vpnconfigurationValue2.Type != JTokenType.Null)
                                        {
                                            VirtualNetworkConfiguration vpnconfigurationInstance2 = new VirtualNetworkConfiguration();
                                            additionalRegionInstance.VirtualNetworkConfiguration = vpnconfigurationInstance2;
                                            
                                            JToken vnetidValue2 = vpnconfigurationValue2["vnetid"];
                                            if (vnetidValue2 != null && vnetidValue2.Type != JTokenType.Null)
                                            {
                                                Guid vnetidInstance2 = Guid.Parse(((string)vnetidValue2));
                                                vpnconfigurationInstance2.VnetId = vnetidInstance2;
                                            }
                                            
                                            JToken subnetnameValue2 = vpnconfigurationValue2["subnetname"];
                                            if (subnetnameValue2 != null && subnetnameValue2.Type != JTokenType.Null)
                                            {
                                                string subnetnameInstance2 = ((string)subnetnameValue2);
                                                vpnconfigurationInstance2.SubnetName = subnetnameInstance2;
                                            }
                                            
                                            JToken locationValue4 = vpnconfigurationValue2["location"];
                                            if (locationValue4 != null && locationValue4.Type != JTokenType.Null)
                                            {
                                                string locationInstance4 = ((string)locationValue4);
                                                vpnconfigurationInstance2.Location = locationInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken customPropertiesSequenceElement = ((JToken)propertiesValue["customProperties"]);
                                if (customPropertiesSequenceElement != null && customPropertiesSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property2 in customPropertiesSequenceElement)
                                    {
                                        string customPropertiesKey = ((string)property2.Name);
                                        string customPropertiesValue = ((string)property2.Value);
                                        propertiesInstance.CustomProperties.Add(customPropertiesKey, customPropertiesValue);
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get ApiService Operation Status operation returns the status of
        /// the create or update operation. After calling the operation, you
        /// can call Get ApiService Operation Status to determine whether the
        /// operation has succeeded, failed, or is still in progress. This
        /// method differs GetLongRunningOperationStatus in providing Api
        /// Management service resource description.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> GetApiServiceLongRunningOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetApiServiceLongRunningOperationStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2014-02-14");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceLongRunningOperationResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK || statusCode == HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceLongRunningOperationResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            ApiServiceResource valueInstance = new ApiServiceResource();
                            result.Value = valueInstance;
                            
                            JToken idValue = responseDoc["id"];
                            if (idValue != null && idValue.Type != JTokenType.Null)
                            {
                                string idInstance = ((string)idValue);
                                valueInstance.Id = idInstance;
                            }
                            
                            JToken locationValue = responseDoc["location"];
                            if (locationValue != null && locationValue.Type != JTokenType.Null)
                            {
                                string locationInstance = ((string)locationValue);
                                valueInstance.Location = locationInstance;
                            }
                            
                            JToken nameValue = responseDoc["name"];
                            if (nameValue != null && nameValue.Type != JTokenType.Null)
                            {
                                string nameInstance = ((string)nameValue);
                                valueInstance.Name = nameInstance;
                            }
                            
                            JToken typeValue = responseDoc["type"];
                            if (typeValue != null && typeValue.Type != JTokenType.Null)
                            {
                                string typeInstance = ((string)typeValue);
                                valueInstance.Type = typeInstance;
                            }
                            
                            JToken etagValue = responseDoc["etag"];
                            if (etagValue != null && etagValue.Type != JTokenType.Null)
                            {
                                string etagInstance = ((string)etagValue);
                                valueInstance.ETag = etagInstance;
                            }
                            
                            JToken tagsSequenceElement = ((JToken)responseDoc["tags"]);
                            if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property in tagsSequenceElement)
                                {
                                    string tagsKey = ((string)property.Name);
                                    string tagsValue = ((string)property.Value);
                                    valueInstance.Tags.Add(tagsKey, tagsValue);
                                }
                            }
                            
                            JToken propertiesValue = responseDoc["properties"];
                            if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                            {
                                ApiServiceProperties propertiesInstance = new ApiServiceProperties();
                                valueInstance.Properties = propertiesInstance;
                                
                                JToken publisherEmailValue = propertiesValue["publisherEmail"];
                                if (publisherEmailValue != null && publisherEmailValue.Type != JTokenType.Null)
                                {
                                    string publisherEmailInstance = ((string)publisherEmailValue);
                                    propertiesInstance.PublisherEmail = publisherEmailInstance;
                                }
                                
                                JToken publisherNameValue = propertiesValue["publisherName"];
                                if (publisherNameValue != null && publisherNameValue.Type != JTokenType.Null)
                                {
                                    string publisherNameInstance = ((string)publisherNameValue);
                                    propertiesInstance.PublisherName = publisherNameInstance;
                                }
                                
                                JToken skuValue = propertiesValue["sku"];
                                if (skuValue != null && skuValue.Type != JTokenType.Null)
                                {
                                    ApiServiceSkuProperties skuInstance = new ApiServiceSkuProperties();
                                    propertiesInstance.SkuProperties = skuInstance;
                                    
                                    JToken nameValue2 = skuValue["name"];
                                    if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                    {
                                        SkuType nameInstance2 = ((SkuType)Enum.Parse(typeof(SkuType), ((string)nameValue2), true));
                                        skuInstance.SkuType = nameInstance2;
                                    }
                                    
                                    JToken capacityValue = skuValue["capacity"];
                                    if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                    {
                                        int capacityInstance = ((int)capacityValue);
                                        skuInstance.Capacity = capacityInstance;
                                    }
                                }
                                
                                JToken provisioningStateValue = propertiesValue["provisioningState"];
                                if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                {
                                    string provisioningStateInstance = ((string)provisioningStateValue);
                                    propertiesInstance.ProvisioningState = provisioningStateInstance;
                                }
                                
                                JToken targetProvisioningStateValue = propertiesValue["targetProvisioningState"];
                                if (targetProvisioningStateValue != null && targetProvisioningStateValue.Type != JTokenType.Null)
                                {
                                    string targetProvisioningStateInstance = ((string)targetProvisioningStateValue);
                                    propertiesInstance.TargetProvisioningState = targetProvisioningStateInstance;
                                }
                                
                                JToken createdAtUtcValue = propertiesValue["createdAtUtc"];
                                if (createdAtUtcValue != null && createdAtUtcValue.Type != JTokenType.Null)
                                {
                                    DateTime createdAtUtcInstance = ((DateTime)createdAtUtcValue);
                                    propertiesInstance.CreatedAtUtc = createdAtUtcInstance;
                                }
                                
                                JToken runtimeUrlValue = propertiesValue["runtimeUrl"];
                                if (runtimeUrlValue != null && runtimeUrlValue.Type != JTokenType.Null)
                                {
                                    string runtimeUrlInstance = ((string)runtimeUrlValue);
                                    propertiesInstance.ProxyEndpoint = runtimeUrlInstance;
                                }
                                
                                JToken portalUrlValue = propertiesValue["portalUrl"];
                                if (portalUrlValue != null && portalUrlValue.Type != JTokenType.Null)
                                {
                                    string portalUrlInstance = ((string)portalUrlValue);
                                    propertiesInstance.ManagementPortalEndpoint = portalUrlInstance;
                                }
                                
                                JToken addresserEmailValue = propertiesValue["addresserEmail"];
                                if (addresserEmailValue != null && addresserEmailValue.Type != JTokenType.Null)
                                {
                                    string addresserEmailInstance = ((string)addresserEmailValue);
                                    propertiesInstance.AddresserEmail = addresserEmailInstance;
                                }
                                
                                JToken hostnameConfigurationsArray = propertiesValue["hostnameConfigurations"];
                                if (hostnameConfigurationsArray != null && hostnameConfigurationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken hostnameConfigurationsValue in ((JArray)hostnameConfigurationsArray))
                                    {
                                        HostnameConfiguration hostnameConfigurationInstance = new HostnameConfiguration();
                                        propertiesInstance.HostnameConfigurations.Add(hostnameConfigurationInstance);
                                        
                                        JToken typeValue2 = hostnameConfigurationsValue["type"];
                                        if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                        {
                                            HostnameType typeInstance2 = ((HostnameType)(((int)typeValue2)));
                                            hostnameConfigurationInstance.Type = typeInstance2;
                                        }
                                        
                                        JToken hostnameValue = hostnameConfigurationsValue["hostname"];
                                        if (hostnameValue != null && hostnameValue.Type != JTokenType.Null)
                                        {
                                            string hostnameInstance = ((string)hostnameValue);
                                            hostnameConfigurationInstance.Hostname = hostnameInstance;
                                        }
                                        
                                        JToken certificateValue = hostnameConfigurationsValue["certificate"];
                                        if (certificateValue != null && certificateValue.Type != JTokenType.Null)
                                        {
                                            CertificateInformation certificateInstance = new CertificateInformation();
                                            hostnameConfigurationInstance.Certificate = certificateInstance;
                                            
                                            JToken expiryValue = certificateValue["expiry"];
                                            if (expiryValue != null && expiryValue.Type != JTokenType.Null)
                                            {
                                                DateTime expiryInstance = ((DateTime)expiryValue);
                                                certificateInstance.Expiry = expiryInstance;
                                            }
                                            
                                            JToken thumbprintValue = certificateValue["thumbprint"];
                                            if (thumbprintValue != null && thumbprintValue.Type != JTokenType.Null)
                                            {
                                                string thumbprintInstance = ((string)thumbprintValue);
                                                certificateInstance.Thumbprint = thumbprintInstance;
                                            }
                                            
                                            JToken subjectValue = certificateValue["subject"];
                                            if (subjectValue != null && subjectValue.Type != JTokenType.Null)
                                            {
                                                string subjectInstance = ((string)subjectValue);
                                                certificateInstance.Subject = subjectInstance;
                                            }
                                        }
                                    }
                                }
                                
                                JToken staticIPsArray = propertiesValue["staticIPs"];
                                if (staticIPsArray != null && staticIPsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken staticIPsValue in ((JArray)staticIPsArray))
                                    {
                                        propertiesInstance.StaticIPs.Add(((string)staticIPsValue));
                                    }
                                }
                                
                                JToken vpnconfigurationValue = propertiesValue["vpnconfiguration"];
                                if (vpnconfigurationValue != null && vpnconfigurationValue.Type != JTokenType.Null)
                                {
                                    VirtualNetworkConfiguration vpnconfigurationInstance = new VirtualNetworkConfiguration();
                                    propertiesInstance.VirtualNetworkConfiguration = vpnconfigurationInstance;
                                    
                                    JToken vnetidValue = vpnconfigurationValue["vnetid"];
                                    if (vnetidValue != null && vnetidValue.Type != JTokenType.Null)
                                    {
                                        Guid vnetidInstance = Guid.Parse(((string)vnetidValue));
                                        vpnconfigurationInstance.VnetId = vnetidInstance;
                                    }
                                    
                                    JToken subnetnameValue = vpnconfigurationValue["subnetname"];
                                    if (subnetnameValue != null && subnetnameValue.Type != JTokenType.Null)
                                    {
                                        string subnetnameInstance = ((string)subnetnameValue);
                                        vpnconfigurationInstance.SubnetName = subnetnameInstance;
                                    }
                                    
                                    JToken locationValue2 = vpnconfigurationValue["location"];
                                    if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                                    {
                                        string locationInstance2 = ((string)locationValue2);
                                        vpnconfigurationInstance.Location = locationInstance2;
                                    }
                                }
                                
                                JToken additionalLocationsArray = propertiesValue["additionalLocations"];
                                if (additionalLocationsArray != null && additionalLocationsArray.Type != JTokenType.Null)
                                {
                                    foreach (JToken additionalLocationsValue in ((JArray)additionalLocationsArray))
                                    {
                                        AdditionalRegion additionalRegionInstance = new AdditionalRegion();
                                        propertiesInstance.AdditionalRegions.Add(additionalRegionInstance);
                                        
                                        JToken locationValue3 = additionalLocationsValue["location"];
                                        if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                                        {
                                            string locationInstance3 = ((string)locationValue3);
                                            additionalRegionInstance.Location = locationInstance3;
                                        }
                                        
                                        JToken skuTypeValue = additionalLocationsValue["skuType"];
                                        if (skuTypeValue != null && skuTypeValue.Type != JTokenType.Null)
                                        {
                                            SkuType skuTypeInstance = ((SkuType)Enum.Parse(typeof(SkuType), ((string)skuTypeValue), true));
                                            additionalRegionInstance.SkuType = skuTypeInstance;
                                        }
                                        
                                        JToken skuUnitCountValue = additionalLocationsValue["skuUnitCount"];
                                        if (skuUnitCountValue != null && skuUnitCountValue.Type != JTokenType.Null)
                                        {
                                            int skuUnitCountInstance = ((int)skuUnitCountValue);
                                            additionalRegionInstance.SkuUnitCount = skuUnitCountInstance;
                                        }
                                        
                                        JToken staticIPsArray2 = additionalLocationsValue["staticIPs"];
                                        if (staticIPsArray2 != null && staticIPsArray2.Type != JTokenType.Null)
                                        {
                                            foreach (JToken staticIPsValue2 in ((JArray)staticIPsArray2))
                                            {
                                                additionalRegionInstance.StaticIPs.Add(((string)staticIPsValue2));
                                            }
                                        }
                                        
                                        JToken vpnconfigurationValue2 = additionalLocationsValue["vpnconfiguration"];
                                        if (vpnconfigurationValue2 != null && vpnconfigurationValue2.Type != JTokenType.Null)
                                        {
                                            VirtualNetworkConfiguration vpnconfigurationInstance2 = new VirtualNetworkConfiguration();
                                            additionalRegionInstance.VirtualNetworkConfiguration = vpnconfigurationInstance2;
                                            
                                            JToken vnetidValue2 = vpnconfigurationValue2["vnetid"];
                                            if (vnetidValue2 != null && vnetidValue2.Type != JTokenType.Null)
                                            {
                                                Guid vnetidInstance2 = Guid.Parse(((string)vnetidValue2));
                                                vpnconfigurationInstance2.VnetId = vnetidInstance2;
                                            }
                                            
                                            JToken subnetnameValue2 = vpnconfigurationValue2["subnetname"];
                                            if (subnetnameValue2 != null && subnetnameValue2.Type != JTokenType.Null)
                                            {
                                                string subnetnameInstance2 = ((string)subnetnameValue2);
                                                vpnconfigurationInstance2.SubnetName = subnetnameInstance2;
                                            }
                                            
                                            JToken locationValue4 = vpnconfigurationValue2["location"];
                                            if (locationValue4 != null && locationValue4.Type != JTokenType.Null)
                                            {
                                                string locationInstance4 = ((string)locationValue4);
                                                vpnconfigurationInstance2.Location = locationInstance4;
                                            }
                                        }
                                    }
                                }
                                
                                JToken customPropertiesSequenceElement = ((JToken)propertiesValue["customProperties"]);
                                if (customPropertiesSequenceElement != null && customPropertiesSequenceElement.Type != JTokenType.Null)
                                {
                                    foreach (JProperty property2 in customPropertiesSequenceElement)
                                    {
                                        string customPropertiesKey = ((string)property2.Name);
                                        string customPropertiesValue = ((string)property2.Value);
                                        propertiesInstance.CustomProperties.Add(customPropertiesKey, customPropertiesValue);
                                    }
                                }
                            }
                            
                            ApiManagementError errorInstance = new ApiManagementError();
                            result.Error = errorInstance;
                            
                            JToken codeValue = responseDoc["code"];
                            if (codeValue != null && codeValue.Type != JTokenType.Null)
                            {
                                string codeInstance = ((string)codeValue);
                                errorInstance.Code = codeInstance;
                            }
                            
                            JToken messageValue = responseDoc["message"];
                            if (messageValue != null && messageValue.Type != JTokenType.Null)
                            {
                                string messageInstance = ((string)messageValue);
                                errorInstance.Message = messageInstance;
                            }
                            
                            JToken detailsSequenceElement = ((JToken)responseDoc["details"]);
                            if (detailsSequenceElement != null && detailsSequenceElement.Type != JTokenType.Null)
                            {
                                foreach (JProperty property3 in detailsSequenceElement)
                                {
                                    string detailsKey = ((string)property3.Name);
                                    string detailsValue = ((string)property3.Value);
                                    errorInstance.Details.Add(detailsKey, detailsValue);
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict || statusCode == HttpStatusCode.BadRequest)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The Get Operation Status operation returns the status of the
        /// specified operation. After calling an asynchronous operation, you
        /// can call Get Operation Status to determine whether the operation
        /// has succeeded, failed, or is still in progress.
        /// </summary>
        /// <param name='operationStatusLink'>
        /// Required. Location value returned by the Begin operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response for long running operations.
        /// </returns>
        public async Task<LongRunningOperationResponse> GetLongRunningOperationStatusAsync(string operationStatusLink, CancellationToken cancellationToken)
        {
            // Validate
            if (operationStatusLink == null)
            {
                throw new ArgumentNullException("operationStatusLink");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("operationStatusLink", operationStatusLink);
                TracingAdapter.Enter(invocationId, this, "GetLongRunningOperationStatusAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + operationStatusLink;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2014-02-14");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK && statusCode != HttpStatusCode.Accepted)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    LongRunningOperationResponse result = null;
                    // Deserialize Response
                    result = new LongRunningOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("Location"))
                    {
                        result.OperationStatusLink = httpResponse.Headers.GetValues("Location").FirstOrDefault();
                    }
                    if (httpResponse.Headers.Contains("Retry-After"))
                    {
                        result.RetryAfter = int.Parse(httpResponse.Headers.GetValues("Retry-After").FirstOrDefault(), CultureInfo.InvariantCulture);
                    }
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    if (statusCode == HttpStatusCode.Conflict || statusCode == HttpStatusCode.BadRequest)
                    {
                        result.Status = OperationStatus.Failed;
                    }
                    if (statusCode == HttpStatusCode.OK)
                    {
                        result.Status = OperationStatus.Succeeded;
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets SsoToken for an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the GetSsoToken operation.
        /// </returns>
        public async Task<ApiServiceGetSsoTokenResponse> GetSsoTokenAsync(string resourceGroupName, string name, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                TracingAdapter.Enter(invocationId, this, "GetSsoTokenAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/getssotoken";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceGetSsoTokenResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceGetSsoTokenResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken redirectUriValue = responseDoc["redirect_uri"];
                            if (redirectUriValue != null && redirectUriValue.Type != JTokenType.Null)
                            {
                                string redirectUriInstance = ((string)redirectUriValue);
                                result.RedirectUrl = redirectUriInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// List all Api Management services within a resource group.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Optional. The name of the resource group. If resourceGroupName
        /// value is null the method lists all Api Management services within
        /// subscription
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the List Api Management services operation.
        /// </returns>
        public async Task<ApiServiceListResponse> ListAsync(string resourceGroupName, CancellationToken cancellationToken)
        {
            // Validate
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            if (resourceGroupName != null)
            {
                url = url + "/resourceGroups/" + Uri.EscapeDataString(resourceGroupName);
            }
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceListResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceListResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            JToken valueArray = responseDoc["value"];
                            if (valueArray != null && valueArray.Type != JTokenType.Null)
                            {
                                foreach (JToken valueValue in ((JArray)valueArray))
                                {
                                    ApiServiceResource apiServiceResourceInstance = new ApiServiceResource();
                                    result.Value.Add(apiServiceResourceInstance);
                                    
                                    JToken idValue = valueValue["id"];
                                    if (idValue != null && idValue.Type != JTokenType.Null)
                                    {
                                        string idInstance = ((string)idValue);
                                        apiServiceResourceInstance.Id = idInstance;
                                    }
                                    
                                    JToken locationValue = valueValue["location"];
                                    if (locationValue != null && locationValue.Type != JTokenType.Null)
                                    {
                                        string locationInstance = ((string)locationValue);
                                        apiServiceResourceInstance.Location = locationInstance;
                                    }
                                    
                                    JToken nameValue = valueValue["name"];
                                    if (nameValue != null && nameValue.Type != JTokenType.Null)
                                    {
                                        string nameInstance = ((string)nameValue);
                                        apiServiceResourceInstance.Name = nameInstance;
                                    }
                                    
                                    JToken typeValue = valueValue["type"];
                                    if (typeValue != null && typeValue.Type != JTokenType.Null)
                                    {
                                        string typeInstance = ((string)typeValue);
                                        apiServiceResourceInstance.Type = typeInstance;
                                    }
                                    
                                    JToken etagValue = valueValue["etag"];
                                    if (etagValue != null && etagValue.Type != JTokenType.Null)
                                    {
                                        string etagInstance = ((string)etagValue);
                                        apiServiceResourceInstance.ETag = etagInstance;
                                    }
                                    
                                    JToken tagsSequenceElement = ((JToken)valueValue["tags"]);
                                    if (tagsSequenceElement != null && tagsSequenceElement.Type != JTokenType.Null)
                                    {
                                        foreach (JProperty property in tagsSequenceElement)
                                        {
                                            string tagsKey = ((string)property.Name);
                                            string tagsValue = ((string)property.Value);
                                            apiServiceResourceInstance.Tags.Add(tagsKey, tagsValue);
                                        }
                                    }
                                    
                                    JToken propertiesValue = valueValue["properties"];
                                    if (propertiesValue != null && propertiesValue.Type != JTokenType.Null)
                                    {
                                        ApiServiceProperties propertiesInstance = new ApiServiceProperties();
                                        apiServiceResourceInstance.Properties = propertiesInstance;
                                        
                                        JToken publisherEmailValue = propertiesValue["publisherEmail"];
                                        if (publisherEmailValue != null && publisherEmailValue.Type != JTokenType.Null)
                                        {
                                            string publisherEmailInstance = ((string)publisherEmailValue);
                                            propertiesInstance.PublisherEmail = publisherEmailInstance;
                                        }
                                        
                                        JToken publisherNameValue = propertiesValue["publisherName"];
                                        if (publisherNameValue != null && publisherNameValue.Type != JTokenType.Null)
                                        {
                                            string publisherNameInstance = ((string)publisherNameValue);
                                            propertiesInstance.PublisherName = publisherNameInstance;
                                        }
                                        
                                        JToken skuValue = propertiesValue["sku"];
                                        if (skuValue != null && skuValue.Type != JTokenType.Null)
                                        {
                                            ApiServiceSkuProperties skuInstance = new ApiServiceSkuProperties();
                                            propertiesInstance.SkuProperties = skuInstance;
                                            
                                            JToken nameValue2 = skuValue["name"];
                                            if (nameValue2 != null && nameValue2.Type != JTokenType.Null)
                                            {
                                                SkuType nameInstance2 = ((SkuType)Enum.Parse(typeof(SkuType), ((string)nameValue2), true));
                                                skuInstance.SkuType = nameInstance2;
                                            }
                                            
                                            JToken capacityValue = skuValue["capacity"];
                                            if (capacityValue != null && capacityValue.Type != JTokenType.Null)
                                            {
                                                int capacityInstance = ((int)capacityValue);
                                                skuInstance.Capacity = capacityInstance;
                                            }
                                        }
                                        
                                        JToken provisioningStateValue = propertiesValue["provisioningState"];
                                        if (provisioningStateValue != null && provisioningStateValue.Type != JTokenType.Null)
                                        {
                                            string provisioningStateInstance = ((string)provisioningStateValue);
                                            propertiesInstance.ProvisioningState = provisioningStateInstance;
                                        }
                                        
                                        JToken targetProvisioningStateValue = propertiesValue["targetProvisioningState"];
                                        if (targetProvisioningStateValue != null && targetProvisioningStateValue.Type != JTokenType.Null)
                                        {
                                            string targetProvisioningStateInstance = ((string)targetProvisioningStateValue);
                                            propertiesInstance.TargetProvisioningState = targetProvisioningStateInstance;
                                        }
                                        
                                        JToken createdAtUtcValue = propertiesValue["createdAtUtc"];
                                        if (createdAtUtcValue != null && createdAtUtcValue.Type != JTokenType.Null)
                                        {
                                            DateTime createdAtUtcInstance = ((DateTime)createdAtUtcValue);
                                            propertiesInstance.CreatedAtUtc = createdAtUtcInstance;
                                        }
                                        
                                        JToken runtimeUrlValue = propertiesValue["runtimeUrl"];
                                        if (runtimeUrlValue != null && runtimeUrlValue.Type != JTokenType.Null)
                                        {
                                            string runtimeUrlInstance = ((string)runtimeUrlValue);
                                            propertiesInstance.ProxyEndpoint = runtimeUrlInstance;
                                        }
                                        
                                        JToken portalUrlValue = propertiesValue["portalUrl"];
                                        if (portalUrlValue != null && portalUrlValue.Type != JTokenType.Null)
                                        {
                                            string portalUrlInstance = ((string)portalUrlValue);
                                            propertiesInstance.ManagementPortalEndpoint = portalUrlInstance;
                                        }
                                        
                                        JToken addresserEmailValue = propertiesValue["addresserEmail"];
                                        if (addresserEmailValue != null && addresserEmailValue.Type != JTokenType.Null)
                                        {
                                            string addresserEmailInstance = ((string)addresserEmailValue);
                                            propertiesInstance.AddresserEmail = addresserEmailInstance;
                                        }
                                        
                                        JToken hostnameConfigurationsArray = propertiesValue["hostnameConfigurations"];
                                        if (hostnameConfigurationsArray != null && hostnameConfigurationsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken hostnameConfigurationsValue in ((JArray)hostnameConfigurationsArray))
                                            {
                                                HostnameConfiguration hostnameConfigurationInstance = new HostnameConfiguration();
                                                propertiesInstance.HostnameConfigurations.Add(hostnameConfigurationInstance);
                                                
                                                JToken typeValue2 = hostnameConfigurationsValue["type"];
                                                if (typeValue2 != null && typeValue2.Type != JTokenType.Null)
                                                {
                                                    HostnameType typeInstance2 = ((HostnameType)(((int)typeValue2)));
                                                    hostnameConfigurationInstance.Type = typeInstance2;
                                                }
                                                
                                                JToken hostnameValue = hostnameConfigurationsValue["hostname"];
                                                if (hostnameValue != null && hostnameValue.Type != JTokenType.Null)
                                                {
                                                    string hostnameInstance = ((string)hostnameValue);
                                                    hostnameConfigurationInstance.Hostname = hostnameInstance;
                                                }
                                                
                                                JToken certificateValue = hostnameConfigurationsValue["certificate"];
                                                if (certificateValue != null && certificateValue.Type != JTokenType.Null)
                                                {
                                                    CertificateInformation certificateInstance = new CertificateInformation();
                                                    hostnameConfigurationInstance.Certificate = certificateInstance;
                                                    
                                                    JToken expiryValue = certificateValue["expiry"];
                                                    if (expiryValue != null && expiryValue.Type != JTokenType.Null)
                                                    {
                                                        DateTime expiryInstance = ((DateTime)expiryValue);
                                                        certificateInstance.Expiry = expiryInstance;
                                                    }
                                                    
                                                    JToken thumbprintValue = certificateValue["thumbprint"];
                                                    if (thumbprintValue != null && thumbprintValue.Type != JTokenType.Null)
                                                    {
                                                        string thumbprintInstance = ((string)thumbprintValue);
                                                        certificateInstance.Thumbprint = thumbprintInstance;
                                                    }
                                                    
                                                    JToken subjectValue = certificateValue["subject"];
                                                    if (subjectValue != null && subjectValue.Type != JTokenType.Null)
                                                    {
                                                        string subjectInstance = ((string)subjectValue);
                                                        certificateInstance.Subject = subjectInstance;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken staticIPsArray = propertiesValue["staticIPs"];
                                        if (staticIPsArray != null && staticIPsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken staticIPsValue in ((JArray)staticIPsArray))
                                            {
                                                propertiesInstance.StaticIPs.Add(((string)staticIPsValue));
                                            }
                                        }
                                        
                                        JToken vpnconfigurationValue = propertiesValue["vpnconfiguration"];
                                        if (vpnconfigurationValue != null && vpnconfigurationValue.Type != JTokenType.Null)
                                        {
                                            VirtualNetworkConfiguration vpnconfigurationInstance = new VirtualNetworkConfiguration();
                                            propertiesInstance.VirtualNetworkConfiguration = vpnconfigurationInstance;
                                            
                                            JToken vnetidValue = vpnconfigurationValue["vnetid"];
                                            if (vnetidValue != null && vnetidValue.Type != JTokenType.Null)
                                            {
                                                Guid vnetidInstance = Guid.Parse(((string)vnetidValue));
                                                vpnconfigurationInstance.VnetId = vnetidInstance;
                                            }
                                            
                                            JToken subnetnameValue = vpnconfigurationValue["subnetname"];
                                            if (subnetnameValue != null && subnetnameValue.Type != JTokenType.Null)
                                            {
                                                string subnetnameInstance = ((string)subnetnameValue);
                                                vpnconfigurationInstance.SubnetName = subnetnameInstance;
                                            }
                                            
                                            JToken locationValue2 = vpnconfigurationValue["location"];
                                            if (locationValue2 != null && locationValue2.Type != JTokenType.Null)
                                            {
                                                string locationInstance2 = ((string)locationValue2);
                                                vpnconfigurationInstance.Location = locationInstance2;
                                            }
                                        }
                                        
                                        JToken additionalLocationsArray = propertiesValue["additionalLocations"];
                                        if (additionalLocationsArray != null && additionalLocationsArray.Type != JTokenType.Null)
                                        {
                                            foreach (JToken additionalLocationsValue in ((JArray)additionalLocationsArray))
                                            {
                                                AdditionalRegion additionalRegionInstance = new AdditionalRegion();
                                                propertiesInstance.AdditionalRegions.Add(additionalRegionInstance);
                                                
                                                JToken locationValue3 = additionalLocationsValue["location"];
                                                if (locationValue3 != null && locationValue3.Type != JTokenType.Null)
                                                {
                                                    string locationInstance3 = ((string)locationValue3);
                                                    additionalRegionInstance.Location = locationInstance3;
                                                }
                                                
                                                JToken skuTypeValue = additionalLocationsValue["skuType"];
                                                if (skuTypeValue != null && skuTypeValue.Type != JTokenType.Null)
                                                {
                                                    SkuType skuTypeInstance = ((SkuType)Enum.Parse(typeof(SkuType), ((string)skuTypeValue), true));
                                                    additionalRegionInstance.SkuType = skuTypeInstance;
                                                }
                                                
                                                JToken skuUnitCountValue = additionalLocationsValue["skuUnitCount"];
                                                if (skuUnitCountValue != null && skuUnitCountValue.Type != JTokenType.Null)
                                                {
                                                    int skuUnitCountInstance = ((int)skuUnitCountValue);
                                                    additionalRegionInstance.SkuUnitCount = skuUnitCountInstance;
                                                }
                                                
                                                JToken staticIPsArray2 = additionalLocationsValue["staticIPs"];
                                                if (staticIPsArray2 != null && staticIPsArray2.Type != JTokenType.Null)
                                                {
                                                    foreach (JToken staticIPsValue2 in ((JArray)staticIPsArray2))
                                                    {
                                                        additionalRegionInstance.StaticIPs.Add(((string)staticIPsValue2));
                                                    }
                                                }
                                                
                                                JToken vpnconfigurationValue2 = additionalLocationsValue["vpnconfiguration"];
                                                if (vpnconfigurationValue2 != null && vpnconfigurationValue2.Type != JTokenType.Null)
                                                {
                                                    VirtualNetworkConfiguration vpnconfigurationInstance2 = new VirtualNetworkConfiguration();
                                                    additionalRegionInstance.VirtualNetworkConfiguration = vpnconfigurationInstance2;
                                                    
                                                    JToken vnetidValue2 = vpnconfigurationValue2["vnetid"];
                                                    if (vnetidValue2 != null && vnetidValue2.Type != JTokenType.Null)
                                                    {
                                                        Guid vnetidInstance2 = Guid.Parse(((string)vnetidValue2));
                                                        vpnconfigurationInstance2.VnetId = vnetidInstance2;
                                                    }
                                                    
                                                    JToken subnetnameValue2 = vpnconfigurationValue2["subnetname"];
                                                    if (subnetnameValue2 != null && subnetnameValue2.Type != JTokenType.Null)
                                                    {
                                                        string subnetnameInstance2 = ((string)subnetnameValue2);
                                                        vpnconfigurationInstance2.SubnetName = subnetnameInstance2;
                                                    }
                                                    
                                                    JToken locationValue4 = vpnconfigurationValue2["location"];
                                                    if (locationValue4 != null && locationValue4.Type != JTokenType.Null)
                                                    {
                                                        string locationInstance4 = ((string)locationValue4);
                                                        vpnconfigurationInstance2.Location = locationInstance4;
                                                    }
                                                }
                                            }
                                        }
                                        
                                        JToken customPropertiesSequenceElement = ((JToken)propertiesValue["customProperties"]);
                                        if (customPropertiesSequenceElement != null && customPropertiesSequenceElement.Type != JTokenType.Null)
                                        {
                                            foreach (JProperty property2 in customPropertiesSequenceElement)
                                            {
                                                string customPropertiesKey = ((string)property2.Name);
                                                string customPropertiesValue = ((string)property2.Value);
                                                propertiesInstance.CustomProperties.Add(customPropertiesKey, customPropertiesValue);
                                            }
                                        }
                                    }
                                }
                            }
                            
                            JToken nextLinkValue = responseDoc["nextLink"];
                            if (nextLinkValue != null && nextLinkValue.Type != JTokenType.Null)
                            {
                                string nextLinkInstance = ((string)nextLinkValue);
                                result.NextLink = nextLinkInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Manage (CUD) deployments of an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the ManageDeployments operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> ManageDeploymentsAsync(string resourceGroupName, string name, ApiServiceManageDeploymentsParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "ManageDeploymentsAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginManagingDeploymentsAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Manage (CUD) VPN configuration of an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the ManageVirtualNetworks
        /// operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> ManageVirtualNetworksAsync(string resourceGroupName, string name, ApiServiceManageVirtualNetworksParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "ManageVirtualNetworksAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginManagingVirtualNetworksAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Restore an Api Management service from backup.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Restore Api Management service
        /// from backup operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> RestoreAsync(string resourceGroupName, string name, ApiServiceBackupRestoreParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "RestoreAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginRestoringAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Update hostname of an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the UpdateHostname operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the CreateOrUpdate Api Management service long
        /// running operation.
        /// </returns>
        public async Task<ApiServiceLongRunningOperationResponse> UpdateHostnameAsync(string resourceGroupName, string name, ApiServiceUpdateHostnameParameters parameters, CancellationToken cancellationToken)
        {
            ApiManagementClient client = this.Client;
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "UpdateHostnameAsync", tracingParameters);
            }
            
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse response = await client.ResourceProvider.BeginUpdatingHostnameAsync(resourceGroupName, name, parameters, cancellationToken).ConfigureAwait(false);
            if (response.Status == OperationStatus.Succeeded)
            {
                return response;
            }
            cancellationToken.ThrowIfCancellationRequested();
            ApiServiceLongRunningOperationResponse result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
            int delayInSeconds = response.RetryAfter;
            if (delayInSeconds == 0)
            {
                delayInSeconds = 60;
            }
            if (client.LongRunningOperationInitialTimeout >= 0)
            {
                delayInSeconds = client.LongRunningOperationInitialTimeout;
            }
            while ((result.Status != OperationStatus.InProgress) == false)
            {
                cancellationToken.ThrowIfCancellationRequested();
                await TaskEx.Delay(delayInSeconds * 1000, cancellationToken).ConfigureAwait(false);
                cancellationToken.ThrowIfCancellationRequested();
                result = await client.ResourceProvider.GetApiServiceLongRunningOperationStatusAsync(response.OperationStatusLink, cancellationToken).ConfigureAwait(false);
                delayInSeconds = result.RetryAfter;
                if (delayInSeconds == 0)
                {
                    delayInSeconds = 60;
                }
                if (client.LongRunningOperationRetryTimeout >= 0)
                {
                    delayInSeconds = client.LongRunningOperationRetryTimeout;
                }
            }
            
            if (shouldTrace)
            {
                TracingAdapter.Exit(invocationId, result);
            }
            
            return result;
        }
        
        /// <summary>
        /// Upload SSL certificate for an Api Management service.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// Required. The name of the resource group.
        /// </param>
        /// <param name='name'>
        /// Required. The name of the Api Management service.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Upload SSL certificate for an
        /// Api Management service operation.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The response of the Upload SSL certificate for an Api Management
        /// service operation.
        /// </returns>
        public async Task<ApiServiceUploadCertificateResponse> UploadCertificateAsync(string resourceGroupName, string name, ApiServiceUploadCertificateParameters parameters, CancellationToken cancellationToken)
        {
            // Validate
            if (resourceGroupName == null)
            {
                throw new ArgumentNullException("resourceGroupName");
            }
            if (name == null)
            {
                throw new ArgumentNullException("name");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (parameters.CertificatePassword == null)
            {
                throw new ArgumentNullException("parameters.CertificatePassword");
            }
            if (parameters.EncodedCertificate == null)
            {
                throw new ArgumentNullException("parameters.EncodedCertificate");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("resourceGroupName", resourceGroupName);
                tracingParameters.Add("name", name);
                tracingParameters.Add("parameters", parameters);
                TracingAdapter.Enter(invocationId, this, "UploadCertificateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/subscriptions/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/resourceGroups/";
            url = url + Uri.EscapeDataString(resourceGroupName);
            url = url + "/providers/";
            url = url + "Microsoft.ApiManagement";
            url = url + "/service/";
            url = url + Uri.EscapeDataString(name);
            url = url + "/updatecertificate";
            List<string> queryParameters = new List<string>();
            queryParameters.Add("api-version=2014-02-14");
            if (queryParameters.Count > 0)
            {
                url = url + "?" + string.Join("&", queryParameters);
            }
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Post;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                JToken requestDoc = null;
                
                JObject apiServiceUploadCertificateParametersValue = new JObject();
                requestDoc = apiServiceUploadCertificateParametersValue;
                
                apiServiceUploadCertificateParametersValue["type"] = ((int)parameters.Type);
                
                apiServiceUploadCertificateParametersValue["certificate"] = parameters.EncodedCertificate;
                
                apiServiceUploadCertificateParametersValue["certificate_password"] = parameters.CertificatePassword;
                
                requestContent = requestDoc.ToString(Newtonsoft.Json.Formatting.Indented);
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/json");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ApiServiceUploadCertificateResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ApiServiceUploadCertificateResponse();
                        JToken responseDoc = null;
                        if (string.IsNullOrEmpty(responseContent) == false)
                        {
                            responseDoc = JToken.Parse(responseContent);
                        }
                        
                        if (responseDoc != null && responseDoc.Type != JTokenType.Null)
                        {
                            CertificateInformation valueInstance = new CertificateInformation();
                            result.Value = valueInstance;
                            
                            JToken expiryValue = responseDoc["expiry"];
                            if (expiryValue != null && expiryValue.Type != JTokenType.Null)
                            {
                                DateTime expiryInstance = ((DateTime)expiryValue);
                                valueInstance.Expiry = expiryInstance;
                            }
                            
                            JToken thumbprintValue = responseDoc["thumbprint"];
                            if (thumbprintValue != null && thumbprintValue.Type != JTokenType.Null)
                            {
                                string thumbprintInstance = ((string)thumbprintValue);
                                valueInstance.Thumbprint = thumbprintInstance;
                            }
                            
                            JToken subjectValue = responseDoc["subject"];
                            if (subjectValue != null && subjectValue.Type != JTokenType.Null)
                            {
                                string subjectInstance = ((string)subjectValue);
                                valueInstance.Subject = subjectInstance;
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
