// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.WebSites;
using Microsoft.WindowsAzure.Management.WebSites.Models;

namespace Microsoft.WindowsAzure.Management.WebSites
{
    /// <summary>
    /// The Web Sites Management API provides a RESTful set of web services
    /// that interact with the Windows Azure Web Sites service to manage your
    /// web sites. The API has entities that capture the relationship between
    /// an end user and Windows Azure Web Sites service.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx for
    /// more information)
    /// </summary>
    public static partial class WebSiteOperationsExtensions
    {
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='targetSwapSlot'>
        /// Required. The name of the target slot to be swapped with.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse ApplySlotConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string targetSwapSlot)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).ApplySlotConfigurationAsync(webSpaceName, webSiteName, targetSwapSlot);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='targetSwapSlot'>
        /// Required. The name of the target slot to be swapped with.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> ApplySlotConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string targetSwapSlot)
        {
            return operations.ApplySlotConfigurationAsync(webSpaceName, webSiteName, targetSwapSlot, CancellationToken.None);
        }
        
        /// <summary>
        /// Backups a site on-demand.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='backupRequest'>
        /// Required. A backup specification.
        /// </param>
        /// <returns>
        /// The backup record created based on the backup request.
        /// </returns>
        public static WebSiteBackupResponse Backup(this IWebSiteOperations operations, string webSpaceName, string webSiteName, BackupRequest backupRequest)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).BackupAsync(webSpaceName, webSiteName, backupRequest);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Backups a site on-demand.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='backupRequest'>
        /// Required. A backup specification.
        /// </param>
        /// <returns>
        /// The backup record created based on the backup request.
        /// </returns>
        public static Task<WebSiteBackupResponse> BackupAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, BackupRequest backupRequest)
        {
            return operations.BackupAsync(webSpaceName, webSiteName, backupRequest, CancellationToken.None);
        }
        
        /// <summary>
        /// You can swap a web site from one slot to another slot.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='sourceSlotName'>
        /// Required. The name of the first web site slot to swap (source).
        /// </param>
        /// <param name='targetSlotName'>
        /// Required. The name of the second web site slot to swap with
        /// (target).
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public static WebSiteOperationStatusResponse BeginSwappingSlots(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string sourceSlotName, string targetSlotName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).BeginSwappingSlotsAsync(webSpaceName, webSiteName, sourceSlotName, targetSlotName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can swap a web site from one slot to another slot.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='sourceSlotName'>
        /// Required. The name of the first web site slot to swap (source).
        /// </param>
        /// <param name='targetSlotName'>
        /// Required. The name of the second web site slot to swap with
        /// (target).
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public static Task<WebSiteOperationStatusResponse> BeginSwappingSlotsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string sourceSlotName, string targetSlotName)
        {
            return operations.BeginSwappingSlotsAsync(webSpaceName, webSiteName, sourceSlotName, targetSlotName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can create a web site by using a POST request that includes the
        /// name of the web site and other information in the request body.
        /// (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166986.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Web Site operation.
        /// </param>
        /// <returns>
        /// The Create Web Site operation response.
        /// </returns>
        public static WebSiteCreateResponse Create(this IWebSiteOperations operations, string webSpaceName, WebSiteCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).CreateAsync(webSpaceName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can create a web site by using a POST request that includes the
        /// name of the web site and other information in the request body.
        /// (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166986.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Web Site operation.
        /// </param>
        /// <returns>
        /// The Create Web Site operation response.
        /// </returns>
        public static Task<WebSiteCreateResponse> CreateAsync(this IWebSiteOperations operations, string webSpaceName, WebSiteCreateParameters parameters)
        {
            return operations.CreateAsync(webSpaceName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Creates an association to a hybrid connection for a web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='siteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Hybrid Connection
        /// operation.
        /// </param>
        /// <returns>
        /// The Create Hybrid Connection operation response.
        /// </returns>
        public static HybridConnectionCreateResponse CreateHybridConnection(this IWebSiteOperations operations, string webSpaceName, string siteName, HybridConnectionCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).CreateHybridConnectionAsync(webSpaceName, siteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Creates an association to a hybrid connection for a web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='siteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Hybrid Connection
        /// operation.
        /// </param>
        /// <returns>
        /// The Create Hybrid Connection operation response.
        /// </returns>
        public static Task<HybridConnectionCreateResponse> CreateHybridConnectionAsync(this IWebSiteOperations operations, string webSpaceName, string siteName, HybridConnectionCreateParameters parameters)
        {
            return operations.CreateHybridConnectionAsync(webSpaceName, siteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository create operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse CreateRepository(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).CreateRepositoryAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository create operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> CreateRepositoryAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.CreateRepositoryAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can delete a web site by issuing an HTTP DELETE request. If the
        /// web site being deleted is the only site remaining in a server
        /// farm, you can optionally delete the server farm as well by using
        /// the deleteEmptyServerFarm parameter.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236430.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Delete Web Site operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Delete(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteDeleteParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).DeleteAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can delete a web site by issuing an HTTP DELETE request. If the
        /// web site being deleted is the only site remaining in a server
        /// farm, you can optionally delete the server farm as well by using
        /// the deleteEmptyServerFarm parameter.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236430.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Delete Web Site operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteDeleteParameters parameters)
        {
            return operations.DeleteAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Deletes a hybrid connection on a specific site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='hybridConnectionName'>
        /// Required. The name of the hybrid connection entity
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse DeleteHybridConnection(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string hybridConnectionName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).DeleteHybridConnectionAsync(webSpaceName, webSiteName, hybridConnectionName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Deletes a hybrid connection on a specific site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='hybridConnectionName'>
        /// Required. The name of the hybrid connection entity
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> DeleteHybridConnectionAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string hybridConnectionName)
        {
            return operations.DeleteHybridConnectionAsync(webSpaceName, webSiteName, hybridConnectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository delete operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Delete Repository Web Site operation response.
        /// </returns>
        public static WebSiteDeleteRepositoryResponse DeleteRepository(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).DeleteRepositoryAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository delete operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Delete Repository Web Site operation response.
        /// </returns>
        public static Task<WebSiteDeleteRepositoryResponse> DeleteRepositoryAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.DeleteRepositoryAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// Scans a backup in a storage account and returns database
        /// information etc. Should be called before calling Restore to
        /// discover what parameters are needed for the restore operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='restoreRequest'>
        /// Required. A restore request.
        /// </param>
        /// <returns>
        /// The information gathered about a backup storaged in a storage
        /// account.
        /// </returns>
        public static WebSiteRestoreDiscoverResponse Discover(this IWebSiteOperations operations, string webSpaceName, string webSiteName, RestoreRequest restoreRequest)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).DiscoverAsync(webSpaceName, webSiteName, restoreRequest);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Scans a backup in a storage account and returns database
        /// information etc. Should be called before calling Restore to
        /// discover what parameters are needed for the restore operation.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='restoreRequest'>
        /// Required. A restore request.
        /// </param>
        /// <returns>
        /// The information gathered about a backup storaged in a storage
        /// account.
        /// </returns>
        public static Task<WebSiteRestoreDiscoverResponse> DiscoverAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, RestoreRequest restoreRequest)
        {
            return operations.DiscoverAsync(webSpaceName, webSiteName, restoreRequest, CancellationToken.None);
        }
        
        /// <summary>
        /// You can generate a new random password for publishing a site by
        /// issuing an HTTP POST request. Tip: If you want to verify that the
        /// publish password has changed, issue an HTTP GET on /publishxml
        /// before calling /newpassword. In the publish XML, note the hash
        /// value in the userPWD attribute. After calling /newpassword, call
        /// /publishxml again. You can then compare the new value of userPWD
        /// in the Publish XML with the one you noted earlier.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236428.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse GeneratePassword(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GeneratePasswordAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can generate a new random password for publishing a site by
        /// issuing an HTTP POST request. Tip: If you want to verify that the
        /// publish password has changed, issue an HTTP GET on /publishxml
        /// before calling /newpassword. In the publish XML, note the hash
        /// value in the userPWD attribute. After calling /newpassword, call
        /// /publishxml again. You can then compare the new value of userPWD
        /// in the Publish XML with the one you noted earlier.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236428.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> GeneratePasswordAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GeneratePasswordAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve details for a web site by issuing an HTTP GET
        /// request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167007.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Parameters supplied to the Get Web Site Operation.
        /// </param>
        /// <returns>
        /// The Get Web Site operation response.
        /// </returns>
        public static WebSiteGetResponse Get(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteGetParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve details for a web site by issuing an HTTP GET
        /// request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167007.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Optional. Parameters supplied to the Get Web Site Operation.
        /// </param>
        /// <returns>
        /// The Get Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetResponse> GetAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteGetParameters parameters)
        {
            return operations.GetAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets a schedule configuration for site backups.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// Scheduled backup definition.
        /// </returns>
        public static WebSiteGetBackupConfigurationResponse GetBackupConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetBackupConfigurationAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets a schedule configuration for site backups.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// Scheduled backup definition.
        /// </returns>
        public static Task<WebSiteGetBackupConfigurationResponse> GetBackupConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetBackupConfigurationAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve the config settings for a web site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Configuration Web Site operation response.
        /// </returns>
        public static WebSiteGetConfigurationResponse GetConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetConfigurationAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve the config settings for a web site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Configuration Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetConfigurationResponse> GetConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetConfigurationAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve historical usage metrics for a site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166964.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Get Historical Usage Metrics
        /// Web Site operation.
        /// </param>
        /// <returns>
        /// The Get Historical Usage Metrics Web Site operation response.
        /// </returns>
        public static WebSiteGetHistoricalUsageMetricsResponse GetHistoricalUsageMetrics(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteGetHistoricalUsageMetricsParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetHistoricalUsageMetricsAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve historical usage metrics for a site by issuing an
        /// HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166964.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Get Historical Usage Metrics
        /// Web Site operation.
        /// </param>
        /// <returns>
        /// The Get Historical Usage Metrics Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetHistoricalUsageMetricsResponse> GetHistoricalUsageMetricsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteGetHistoricalUsageMetricsParameters parameters)
        {
            return operations.GetHistoricalUsageMetricsAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieves a particular hybrid connection that belongs to a specific
        /// site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='hybridConnectionName'>
        /// Required. The name of the hybrid connection entity
        /// </param>
        /// <returns>
        /// The Get Hybrid Connection operation response.
        /// </returns>
        public static HybridConnectionGetResponse GetHybridConnection(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string hybridConnectionName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetHybridConnectionAsync(webSpaceName, webSiteName, hybridConnectionName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieves a particular hybrid connection that belongs to a specific
        /// site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='hybridConnectionName'>
        /// Required. The name of the hybrid connection entity
        /// </param>
        /// <returns>
        /// The Get Hybrid Connection operation response.
        /// </returns>
        public static Task<HybridConnectionGetResponse> GetHybridConnectionAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string hybridConnectionName)
        {
            return operations.GetHybridConnectionAsync(webSpaceName, webSiteName, hybridConnectionName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve the list of active instances by ids for a web site
        /// by issuing an HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The web site instance ids response.
        /// </returns>
        public static WebSiteInstanceIdsResponse GetInstanceIds(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetInstanceIdsAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve the list of active instances by ids for a web site
        /// by issuing an HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166981.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The web site instance ids response.
        /// </returns>
        public static Task<WebSiteInstanceIdsResponse> GetInstanceIdsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetInstanceIdsAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve the publish settings information for a web site by
        /// issuing an HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166996.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Publish Profile Web Site operation response.
        /// </returns>
        public static WebSiteGetPublishProfileResponse GetPublishProfile(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetPublishProfileAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve the publish settings information for a web site by
        /// issuing an HTTP GET request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166996.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Publish Profile Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetPublishProfileResponse> GetPublishProfileAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetPublishProfileAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository get operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Repository Web Site operation response.
        /// </returns>
        public static WebSiteGetRepositoryResponse GetRepository(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetRepositoryAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository get operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Repository Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetRepositoryResponse> GetRepositoryAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetRepositoryAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static SlotConfigNames GetSlotConfigNames(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetSlotConfigNamesAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<SlotConfigNames> GetSlotConfigNamesAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetSlotConfigNamesAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can retrieve current usage metrics for a site by issuing an
        /// HTTP GET request. The metrics returned include CPU Time, Data In,
        /// Data Out, Local Bytes Read, Local Bytes Written, Network Bytes
        /// Read, Network Bytes Written, WP Stop Requests, Memory Usage, CPU
        /// Time - Minute Limit, and File System Storage.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166991.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Usage Metrics Web Site operation response.
        /// </returns>
        public static WebSiteGetUsageMetricsResponse GetUsageMetrics(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).GetUsageMetricsAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can retrieve current usage metrics for a site by issuing an
        /// HTTP GET request. The metrics returned include CPU Time, Data In,
        /// Data Out, Local Bytes Read, Local Bytes Written, Network Bytes
        /// Read, Network Bytes Written, WP Stop Requests, Memory Usage, CPU
        /// Time - Minute Limit, and File System Storage.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166991.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Get Usage Metrics Web Site operation response.
        /// </returns>
        public static Task<WebSiteGetUsageMetricsResponse> GetUsageMetricsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.GetUsageMetricsAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// Determines if a host name is available.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Is Hostname Available Web Site operation response.
        /// </returns>
        public static WebSiteIsHostnameAvailableResponse IsHostnameAvailable(this IWebSiteOperations operations, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).IsHostnameAvailableAsync(webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Determines if a host name is available.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The Is Hostname Available Web Site operation response.
        /// </returns>
        public static Task<WebSiteIsHostnameAvailableResponse> IsHostnameAvailableAsync(this IWebSiteOperations operations, string webSiteName)
        {
            return operations.IsHostnameAvailableAsync(webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// Returns list of all backups which are tracked by the system.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// List of backups for the website.
        /// </returns>
        public static WebSiteGetBackupsResponse ListBackups(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).ListBackupsAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Returns list of all backups which are tracked by the system.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// List of backups for the website.
        /// </returns>
        public static Task<WebSiteGetBackupsResponse> ListBackupsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.ListBackupsAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// Retrieves a list of all hybrid connections on a specific web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The List Hybrid Connection operation response.
        /// </returns>
        public static HybridConnectionListResponse ListHybridConnections(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).ListHybridConnectionsAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Retrieves a list of all hybrid connections on a specific web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// The List Hybrid Connection operation response.
        /// </returns>
        public static Task<HybridConnectionListResponse> ListHybridConnectionsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.ListHybridConnectionsAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse ResetSlotConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).ResetSlotConfigurationAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> ResetSlotConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.ResetSlotConfigurationAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can restart a web site by issuing an HTTP POST request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236425.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Restart(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).RestartAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can restart a web site by issuing an HTTP POST request.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn236425.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> RestartAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.RestartAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// Restores a site to either a new site or existing site (Overwrite
        /// flag has to be set to true for that).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='restoreRequest'>
        /// Required. A restore request.
        /// </param>
        /// <returns>
        /// Restore operation information.
        /// </returns>
        public static WebSiteRestoreResponse Restore(this IWebSiteOperations operations, string webSpaceName, string webSiteName, RestoreRequest restoreRequest)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).RestoreAsync(webSpaceName, webSiteName, restoreRequest);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Restores a site to either a new site or existing site (Overwrite
        /// flag has to be set to true for that).
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='restoreRequest'>
        /// Required. A restore request.
        /// </param>
        /// <returns>
        /// Restore operation information.
        /// </returns>
        public static Task<WebSiteRestoreResponse> RestoreAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, RestoreRequest restoreRequest)
        {
            return operations.RestoreAsync(webSpaceName, webSiteName, restoreRequest, CancellationToken.None);
        }
        
        /// <summary>
        /// You can swap a web site from one slot to another slot.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='sourceSlotName'>
        /// Required. The name of the first web site slot to swap (source).
        /// </param>
        /// <param name='targetSlotName'>
        /// Required. The name of the second web site slot to swap with
        /// (target).
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public static WebSiteOperationStatusResponse SwapSlots(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string sourceSlotName, string targetSlotName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).SwapSlotsAsync(webSpaceName, webSiteName, sourceSlotName, targetSlotName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can swap a web site from one slot to another slot.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='sourceSlotName'>
        /// Required. The name of the first web site slot to swap (source).
        /// </param>
        /// <param name='targetSlotName'>
        /// Required. The name of the second web site slot to swap with
        /// (target).
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified long-running
        /// operation, indicating whether it has succeeded, is inprogress, has
        /// timed out, or has failed. Note that this status is distinct from
        /// the HTTP status code returned for the Get Operation Status
        /// operation itself. If the long-running operation failed, the
        /// response body includes error information regarding the failure.
        /// </returns>
        public static Task<WebSiteOperationStatusResponse> SwapSlotsAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, string sourceSlotName, string targetSlotName)
        {
            return operations.SwapSlotsAsync(webSpaceName, webSiteName, sourceSlotName, targetSlotName, CancellationToken.None);
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository sync operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse SyncRepository(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).SyncRepositoryAsync(webSpaceName, webSiteName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// A web site repository is essentially a Git repository that you can
        /// use to manage your web site content. By using Git source control
        /// tools, you can push or pull version-controlled changes to your
        /// site. This API executes a repository sync operation.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn166967.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> SyncRepositoryAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName)
        {
            return operations.SyncRepositoryAsync(webSpaceName, webSiteName, CancellationToken.None);
        }
        
        /// <summary>
        /// You can update the settings for a web site by using the HTTP PUT
        /// method and by specifying the settings in the request body.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167005.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Web Site operation.
        /// </param>
        /// <returns>
        /// The Update Web Site operation response.
        /// </returns>
        public static WebSiteUpdateResponse Update(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).UpdateAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can update the settings for a web site by using the HTTP PUT
        /// method and by specifying the settings in the request body.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/dn167005.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Web Site operation.
        /// </param>
        /// <returns>
        /// The Update Web Site operation response.
        /// </returns>
        public static Task<WebSiteUpdateResponse> UpdateAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteUpdateParameters parameters)
        {
            return operations.UpdateAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Updates a backup schedule for a site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='backupRequest'>
        /// Required. A backup schedule specification.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UpdateBackupConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName, BackupRequest backupRequest)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).UpdateBackupConfigurationAsync(webSpaceName, webSiteName, backupRequest);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Updates a backup schedule for a site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='backupRequest'>
        /// Required. A backup schedule specification.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateBackupConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, BackupRequest backupRequest)
        {
            return operations.UpdateBackupConfigurationAsync(webSpaceName, webSiteName, backupRequest, CancellationToken.None);
        }
        
        /// <summary>
        /// You can update the config settings for a web site by issuing an
        /// HTTP PUT with a request body containing the updated settings.
        /// (see http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Configuration Web Site
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UpdateConfiguration(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteUpdateConfigurationParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).UpdateConfigurationAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// You can update the config settings for a web site by issuing an
        /// HTTP PUT with a request body containing the updated settings.
        /// (see http://msdn.microsoft.com/en-us/library/windowsazure/dn166985.aspx
        /// for more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Configuration Web Site
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateConfigurationAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, WebSiteUpdateConfigurationParameters parameters)
        {
            return operations.UpdateConfigurationAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Updates an association to a hybrid connection for a web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='siteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Hybrid Connection
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UpdateHybridConnection(this IWebSiteOperations operations, string webSpaceName, string siteName, HybridConnectionUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).UpdateHybridConnectionAsync(webSpaceName, siteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Updates an association to a hybrid connection for a web site.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='siteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Hybrid Connection
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateHybridConnectionAsync(this IWebSiteOperations operations, string webSpaceName, string siteName, HybridConnectionUpdateParameters parameters)
        {
            return operations.UpdateHybridConnectionAsync(webSpaceName, siteName, parameters, CancellationToken.None);
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse UpdateSlotConfigNames(this IWebSiteOperations operations, string webSpaceName, string webSiteName, SlotConfigNamesUpdate parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IWebSiteOperations)s).UpdateSlotConfigNamesAsync(webSpaceName, webSiteName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.WebSites.IWebSiteOperations.
        /// </param>
        /// <param name='webSpaceName'>
        /// Required. The name of the web space.
        /// </param>
        /// <param name='webSiteName'>
        /// Required. The name of the web site.
        /// </param>
        /// <param name='parameters'>
        /// Required. The parameters.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateSlotConfigNamesAsync(this IWebSiteOperations operations, string webSpaceName, string webSiteName, SlotConfigNamesUpdate parameters)
        {
            return operations.UpdateSlotConfigNamesAsync(webSpaceName, webSiteName, parameters, CancellationToken.None);
        }
    }
}
